#include <iostream>
#include <fstream>
#include <list>
#include <vector>

#include <stdlib.h>
#include <string.h>

#include "thread.h"
#include "controller.h"
#include "plugin.h"
#include "parameters.h"

#include "hdrshop.h"

////////////////////////////////////////////////////////////////////////////////
// Prototypes.
void PrintInterface(control::Controller *stm, 
                    const std::vector<ParamDef> &parameters);

void AwaitCompletion(control::Controller *stm);
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
    int success(0);
    std::string            path;
    std::list<std::string> modules;

    // get rid of stderr
    {
        std::string log;
        log = GetPluginName();
        log+= ".log";
        FILE *fp = freopen(log.c_str(), "w", stderr);

        if(!fp)
        {
            abort(); 
        }
    }

    // Get module path.
    {
        char *p = getenv("STM_MODULE_PATH");

        if(p)
        {
            path = p;
        }
        else
        {
#ifdef _WIN32
            HANDLE hProcess = GetCurrentProcess();
            
            if(hProcess)
            {
                typedef DWORD (WINAPI *tGetModuleFileNameExA)(HANDLE,HMODULE,
                                                              LPTSTR,DWORD);
            
                tGetModuleFileNameExA pGetModuleFileNameExA(0);
            
                HINSTANCE hIns = LoadLibrary("Psapi.dll");
            
                if(hIns)
                {
                    pGetModuleFileNameExA = 
                        (tGetModuleFileNameExA)
                            (::GetProcAddress(hIns,"GetModuleFileNameExA"));
            
                    if(pGetModuleFileNameExA)
                    {
                        char fileName[MAX_PATH];
                        fileName[0] = 0;
            
                        if(pGetModuleFileNameExA(hProcess, 0, fileName,
                                                 sizeof(fileName)))
                        {
                        
                            if(strrchr(fileName, '\\'))
                            {
                                *(strrchr(fileName, '\\')) = 0;
                            }
            
                            strncat(fileName, "\\lib", MAX_PATH);
                            path = fileName;
                        }
                    }
            
                    ::FreeLibrary(hIns);
                }
            }
#endif 
        }
    }

    GetModuleList(modules);

    if(modules.size() == 0)
    {
        fprintf(stderr, "No modules defined.\n");
        success = -1;
    }
    else
    {
        std::vector<std::pair<int, std::string> > parameters;
        int output_id;

        control::Controller *stm(NULL);
        int num_threads = thread::GetProcessorCount();

        thread::ThreadPool::GetInstance()->SetNumThreads(num_threads);
        stm = new control::Controller(path, modules);

        SetupPluginModules(stm, parameters, output_id);
        
        if(argc < 3)
        {
            // Need input & output at least.
            PrintInterface(stm, parameters);
        }
        else
        {
            std::string fn_in, fn_out;
            bool print_interface(false);

            fn_in = argv[1];
            fn_out = argv[2];

            {
                for(int i=0;i<argc;i++)
                {
                    if(strcmp("--help", argv[i]) == 0)
                    {
                        print_interface = true;
                    }
                    else if(strcmp("/?", argv[i]) == 0)
                    {
                        print_interface = true;
                    }
                }
            }

            if(print_interface)
            {
                PrintInterface(stm, parameters);
            }
            else
            {
                if(argc > 3)
                {
                    int i;

                    for(i=2;i<argc;i++)
                    {
                        char * p;
                        p = argv[i];
                        spectral::Parameters *params(NULL);

                        if((*p) == '/')
                        {
                            p++;
                            char val = *p;

                            if(val >= 'A')
                            {
                                p++;
                                if(*p)
                                {
                                    /*if(*p == '=')
                                    {
                                        p++;
                                    }*/
									p++;
                                } 
                                else
                                {
                                    // boolean?
                                }

                                size_t id = (val - 'A');
                                if(id < parameters.size())
                                {
                                    stm->RequestParameters(parameters[id].first);

                                    {
                                        bool received(false);
                                        do
                                        {
                                            control::Message *msg;

                                            msg = stm->GetOutputQueue().Pop();

                                            if(msg)
                                            {
                                                if(msg->GetType() == control::Message::MSG_PARAMETERS)
                                                {
                                                    params = msg->GetParameters();
                                                    received = true;
                                                }
                                                delete msg;
                                            }
                                        } while(!received);
                                    }
                                    if(params)
                                    {
                                        switch(params->GetParameterType(parameters[id].second))
                                        {
                                        case spectral::PARAM_TYPE_INT:
                                            if(p && *p)
                                            {
                                                int n = atoi(p);
                                                params->SetParameter(parameters[id].second, n);
                                            }
                                            break;
                                        case spectral::PARAM_TYPE_FLOAT:
                                            if(p && *p)
                                            {
                                                float n = atof(p);
                                                params->SetParameter(parameters[id].second, n);
                                            }
                                            break;
                                        case spectral::PARAM_TYPE_BOOL:
                                            {
                                                params->SetParameter(parameters[id].second, true);
                                            }
                                        default:
                                            break;
                                        }
                                        stm->SetParameters(parameters[id].first, params);
                                    }

                                }
                            }
                        }
                    }
                }

                stm->LoadImage(0, fn_in);

                SetupPluginLinks(stm);

                stm->Process();

                AwaitCompletion(stm);

                stm->SaveImage(output_id, fn_out);
            }
        }

        stm->Shutdown();
        delete stm;
    }
    
    return success;
}

////////////////////////////////////////////////////////////////////////////////
void PrintInterface(control::Controller *stm, 
                    const std::vector<ParamDef> &parameters)
{
    printf("HDR Shop plugin based on STM version %s: \n", STM_VERSION_STRING);
    printf("%s\n\n", GetPluginDescription());

    printf("HDRSHOPFLAGS:\n");
    printf("HDRSHOPVERSION: 1.0.1\n");
    printf("USAGE: %s [input.pfm] [output.pfm] [options]\n", GetPluginName());
    printf("OPTIONS:\n");

    for(size_t i=0;i<parameters.size();i++)
    {
        spectral::Parameters *params;
        params = NULL;

        stm->RequestParameters(parameters[i].first);

        {
            bool received(false);
            do
            {
                control::Message *msg;

                msg = stm->GetOutputQueue().Pop();

                if(msg)
                {
                    if(msg->GetType() == control::Message::MSG_PARAMETERS)
                    {
                        params = msg->GetParameters();
                        received = true;
                    }
                    delete msg;
                }
            } while(!received);
        }
        if(params)
        {
            char flag_id;
            flag_id = 'A' + (char)i;
            switch(params->GetParameterType(parameters[i].second))
            {
            case spectral::PARAM_TYPE_INT:
                {
                    int32_t min, max, value;
                    params->GetParameter(parameters[i].second, value);
                    params->GetRange(parameters[i].second, min, max);
    
                    printf("\t/%c:[integer=%i]\t%s in [%i, %i]\n", 
                           flag_id, value, parameters[i].second.c_str(),
                           min, max);
                }
                break;
            case spectral::PARAM_TYPE_FLOAT:
                {
                    float min, max, value;
                    params->GetParameter(parameters[i].second, value);
                    params->GetRange(parameters[i].second, min, max);
    
                    printf("\t/%c:[float=%f]\t%s in [%f, %f]\n", 
                           flag_id, value, parameters[i].second.c_str(),
                           min, max);
                }
                break;
            case spectral::PARAM_TYPE_BOOL:
                printf("\t/%c\t\t%s\n", flag_id, parameters[i].second.c_str());
            default:
                break;
            }

            delete params;
        }
    }
}

void AwaitCompletion(control::Controller *stm)
{
    bool cont(true);
    while(cont)
    {
        control::Message *msg;
        msg = stm->GetOutputQueue().Pop();
        if(msg)
        {
            switch(msg->GetType())
            {
            case control::Message::MSG_STOPPED:
                cont = false;
                break;
            case control::Message::MSG_PROGRESS:
                {
                    int id = msg->GetId();
                    float progress = msg->GetFloat();
                    fprintf(stdout, "Job %i %f%% complete.\n", id, progress);
                }
                break;
            case control::Message::MSG_ACTOR_INSTALLED:      // Add a processor actor
                fprintf(stderr, "Installed processor %s in slot %i\n",
                msg->GetString().c_str(), msg->GetId());
                break;
            case control::Message::MSG_ACTOR_REMOVED:      // Add a processor actor
                fprintf(stderr, "Removed processor from slot %i\n", msg->GetId());
                break;
            case control::Message::MSG_ERROR:
                fprintf(stderr, "Error: %s\n", msg->GetString().c_str()); 
                break;
            default:
                break;
            }
            delete msg;
            fflush(stdout);
        }
    }
}

