#include <iostream>
#include <fstream>

#include <stdlib.h>

#include "thread.h"
#include "controller.h"
#include "plugin.h"
#include "parameters.h"

#include "hdrshop.h"

const char *GetPluginDescription(void)
{
    return "Bilateral filter based contrast adjustment.  David Beynon 2011";
}

const char *GetPluginName(void)
{
    return "stm_bilateral_contrast";
}

void GetModuleList(std::list<std::string> &modules)
{
    modules.push_back("pfmloader");
    modules.push_back("pfmsaver");
    modules.push_back("coltolum");
    modules.push_back("bilateral");
    modules.push_back("contrast");
}

void SetupPluginModules(control::Controller *stm,
                        std::vector<ParamDef> &parameters,
                        int &output_id)
{
    // First install processors.
    stm->AddActor(1, "coltolum");
    stm->AddActor(2, "bilateral");
    stm->AddActor(3, "contrast");
    
    // Now set up exported parameter list.

    parameters.push_back(ParamDef(2, std::string("radius")));
    parameters.push_back(ParamDef(2, std::string("passes")));

    parameters.push_back(ParamDef(2, std::string("threshold")));
    parameters.push_back(ParamDef(3, std::string("contrast")));

    parameters.push_back(ParamDef(2, std::string("wrap horizontal")));
    parameters.push_back(ParamDef(2, std::string("wrap vertical")));

    // Specify our output.
    output_id = 3;
}

void SetupPluginLinks(control::Controller *stm)
{
    // Now hook them up.
    stm->AddLink(1, 0, 0);
    stm->AddLink(2, 0, 1);
    stm->AddLink(3, 0, 0);
    stm->AddLink(3, 1, 2);
}
