#include <iostream>
#include <fstream>

#include <stdlib.h>

#include "thread.h"
#include "controller.h"
#include "plugin.h"
#include "parameters.h"

#include "hdrshop.h"

const char *GetPluginDescription(void)
{
    return "Remap 180 degree circular fisheye to equirectangular projection." \
          "  x1,y1 x2,y2 x3,y3 are points on the outside of the \"circle\"." \
          "  Zenith x,y compensates for wobbly photographs." \
          "  Auto size guesses optimal image width from input." \
          "  Samples control anti aliasing." \
          "  Finally the background range gives a percentage of the outside " \
          " of the image averaged together to give bg colour.";
}

const char *GetPluginName(void)
{
    return "stm_remap";
}

void GetModuleList(std::list<std::string> &modules)
{
    modules.push_back("pfmloader");
    modules.push_back("pfmsaver");
    modules.push_back("remap");
}

void SetupPluginModules(control::Controller *stm,
                        std::vector<ParamDef> &parameters,
                        int &output_id)
{
    // First install processors.
    stm->AddActor(1, "remap");
    
    // Now set up exported parameter list.

    parameters.push_back(ParamDef(1, std::string("x1")));
    parameters.push_back(ParamDef(1, std::string("y1")));
    parameters.push_back(ParamDef(1, std::string("x2")));
    parameters.push_back(ParamDef(1, std::string("y2")));
    parameters.push_back(ParamDef(1, std::string("x3")));
    parameters.push_back(ParamDef(1, std::string("y3")));

    parameters.push_back(ParamDef(1, std::string("move zenith")));
    parameters.push_back(ParamDef(1, std::string("zenith x")));
    parameters.push_back(ParamDef(1, std::string("zenith y")));

    parameters.push_back(ParamDef(1, std::string("auto size")));
    parameters.push_back(ParamDef(1, std::string("width")));

    parameters.push_back(ParamDef(1, std::string("samples")));
    parameters.push_back(ParamDef(1, std::string("background range")));

    // Specify our output.
    output_id = 1;
}

void SetupPluginLinks(control::Controller *stm)
{
    // Now hook them up.
    stm->AddLink(1, 0, 0);
}
