// A simple, non thread safe observer mechanism.  For single threaded guis only!
// Use at your own risk.

#ifndef __SIMPLE_OBSERVER_H__
#define __SIMPLE_OBSERVER_H__
#include <map>
#include <set>

class SimpleObserver;
class SimpleObservable;

class SimpleObserverMessage
{
public:
    SimpleObserverMessage(const SimpleObservable *obs)
        : m_observable(obs)
    {
    }
    virtual ~SimpleObserverMessage(void)
    {
    }
    virtual SimpleObserverMessage *Clone(void) const = 0;
    virtual void Deliver(SimpleObserver *) const = 0;

    const SimpleObservable *GetObservable(void) const
    {
        return m_observable;
    }
private:
    const SimpleObservable *m_observable;
};

class SimpleObserver
{
friend class SimpleObservable;
public:
    SimpleObserver()
    {
    }

    virtual ~SimpleObserver();


protected:
    void Register(SimpleObservable *);
    void UnRegister(SimpleObservable *);

    virtual void ReceiveMessage(const SimpleObserverMessage *);

private:
    std::set<SimpleObservable *> m_observing;
};

class SimpleObservable
{
friend class SimpleObserver;
public:

protected:
    void Register(SimpleObserver *observer);
    void UnRegister(SimpleObserver *observer, bool all_instances = false);

    unsigned int SendMessage(const SimpleObserverMessage *) const;

private:
    // Store observer + ref count.  This is a bit "belt & braces", as the observers don't
    // currently keep track, and deregistering should only really be done in the 
    // destructor.
    std::map<SimpleObserver *, unsigned int> m_observers;
};

#endif
