#ifndef __STM_CHROMATICITY_WIDGET_H__
#define __STM_CHROMATICITY_WIDGET_H__

#include <vector>


#include <QWidget>
#include <QDialog>

// Layout the doalog
#include <QGridLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QComboBox>
#include <QImage>

// supported widget types
#include <QLabel>

#include "Config.h"
#include "XyValid.h"

namespace spectral
{
    class Parameters;
    class Processor;
}


class StmXYPane : public QWidget
{
    Q_OBJECT

    class GammaObserver : public QtGuiConfigObserver
    {
    public:
        GammaObserver(StmXYPane *);
        virtual ~GammaObserver()
        {
        }
        virtual void OnSetScreenGamma(float gamma)
        {
            if(m_pane)
            {
                m_pane->setGamma(gamma);
            }
        }

    private:
        StmXYPane *m_pane;
    };

public:
    StmXYPane(QWidget *parent = NULL);

    void setXY(float x, float y);
    void setGamma(float gamma);

signals:
    void chromaticityChanged(float, float);

protected:
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
private:
    bool Validate(float x, float y);
    void MapTo(float &, float &);
    void MapFrom(float &, float &);
    unsigned int IntColour(float, float, float);

    float m_x, m_y, m_gamma;
    QImage m_image;
    XyValid m_validator;
    GammaObserver m_observer;
};

class StmChromaticityDialog : public QDialog
{
    Q_OBJECT

    class WhitePoint
    {
    public:
        WhitePoint(const char *name, float x, float y)
            : m_name(name)
            , m_x(x)
            , m_y(y)
        {
        }

        std::string m_name;
        float       m_x;
        float       m_y;
    };

public:
    StmChromaticityDialog(QWidget *parent = NULL);
    float x(void) const {return m_x;}
    float y(void) const {return m_y;}
    void setXY(float x, float y);

public slots:
    void onChromaticityChanged(float, float);
    void onComboChanged(int);
    void onOk(void);
    void onCancel(void);

signals:
    float chromaticityChanged(float, float);

private:
    void UpdateText(void);

    std::vector<WhitePoint> m_whitePoints;

    float m_x, m_y;
    QWidget     m_buttonHolder;
    QHBoxLayout m_buttonLayout;
    QWidget     m_widgetHolder;
    QVBoxLayout m_widgetLayout;
    QVBoxLayout m_mainLayout;
    QPushButton m_ok, m_cancel;
    StmXYPane   m_pane;
    QWidget     m_textWidgets;
    QHBoxLayout m_textWidgetLayout;
    QComboBox   m_selectCombo;
    QLabel      m_label;    
};


class StmChromaticityWidget : public QPushButton
{
    Q_OBJECT

public:
    StmChromaticityWidget(QWidget *parent = NULL);
    virtual ~StmChromaticityWidget();

    float x(void) const {return m_x;}
    float y(void) const {return m_y;}
    void setXY(float x, float y);

public slots:
    void onPressed(void);
    void onChromaticityChanged(float, float);

signals:
    void chromaticityChanged(float, float);

private:
    void UpdateText(void);
    float m_x, m_y;
    StmChromaticityDialog m_dialog;
};

#endif
