#include <QFileDialog>
#include <QGridLayout>

#include "StmMainWindow.h"
#include "StmWrapper.h"
#include "StmConfigDialog.h"
#include "StmView.h"

#include "controller.h"

StmMainWindow::StmMainWindow(QWidget *parent,
                             const std::string &path,
                             const std::list<std::string> &modules)
    : QWidget(parent)
    , m_wrapper(NULL)
    , m_view(NULL)
    , m_optionsDialog(NULL)
    , m_layout(NULL)
    , m_menuBar(NULL)
    , m_fileMenu(NULL)
    , m_toolsMenu(NULL)
    , m_loadImage(NULL)
    , m_loadProcess(NULL)
    , m_saveProcess(NULL)
    , m_changeOptions(NULL)
{
    m_wrapper = new StmWrapper(path, modules);
    m_optionsDialog = new StmConfigDialog(this);
    m_layout = new QGridLayout(this);

    // Build menu
    m_menuBar = new QMenuBar(this);
    m_fileMenu = m_menuBar->addMenu(QString("&File"));
    m_toolsMenu = m_menuBar->addMenu(QString("&Tools"));

    // File menu 
    m_loadImage = m_fileMenu->addAction(QString("Load &Image"));
    connect(m_loadImage, SIGNAL(triggered()), this, SLOT(loadImage()));
    m_loadProcess = m_fileMenu->addAction(QString("&Load Process"));
    m_saveProcess = m_fileMenu->addAction(QString("&Save Process"));

    // Options menu
    m_changeOptions = m_toolsMenu->addAction(QString("&Options"));
    connect(m_changeOptions, SIGNAL(triggered()), this, SLOT(changeOptions()));

    // Scene
/*
    m_scene = new StmScene(this);
    // connect wrapper messages to scene.
*/
    // view window
    m_view = new StmView(this);

    connect(m_wrapper, SIGNAL(imageData(int, void*)), m_view, SLOT(onImageData(int, void *)));
//    connect(m_wrapper, SIGNAL(imageLoaded(int)), m_view, SLOT(onImageLoaded(int)));
    connect(m_wrapper, SIGNAL(imageLoaded(int)), this, SLOT(onImageLoaded(int)));

    //m_view->setScene(m_scene);
    m_layout->addWidget(m_menuBar, 0, 0);
    m_layout->addWidget(m_view, 1, 0);
}

StmMainWindow::~StmMainWindow()
{
    if(m_wrapper)
    {
        fprintf(stderr, "Shutting down\n");
        m_wrapper->GetController()->Shutdown();
        fprintf(stderr, "Waiting\n");
        m_wrapper->wait();
        delete m_wrapper;
        fprintf(stderr, "Joined\n");
    }

}

void
StmMainWindow::loadImage(void)
{
    QString fn = QFileDialog::getOpenFileName(this, "Load Image", ".", 
                                              "Images (*.png *.exr *.tif)");

    if(fn.length())
    {
        std::string name(fn.toStdString());
        if(m_wrapper && m_wrapper->GetController())
        {
            printf("try to load %s\n", name.c_str());
            m_wrapper->GetController()->LoadImage(name);
        }
    }
}

void
StmMainWindow::changeOptions(void)
{
    if(m_optionsDialog)
    {
        m_optionsDialog->show();
    }
}

void
StmMainWindow::onImageLoaded(int id)
{
    fprintf(stderr,"OnImageLoaded\n");
    if(m_wrapper && m_wrapper->GetController())
    {
        m_wrapper->GetController()->RequestImage(id);
    }
}

void
StmMainWindow::onImageData(int id, void *p)
{
}

