#ifndef __STM_PROCESSOR_DIALOG_H__
#define __STM_PROCESSOR_DIALOG_H__

#include <vector>

#include <QWidget>
#include <QDialog>

// Layout the doalog
#include <QGridLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>

// supported widget types
#include <QLabel>

#include "StmChromaticityWidget.h"

namespace spectral
{
    class Parameters;
    class Processor;
}

class StmParameterForm : public QWidget
{
    Q_OBJECT

public:
    StmParameterForm(QWidget *parent = NULL, spectral::Parameters *parameters = NULL);
    virtual ~StmParameterForm();

    spectral::Parameters *GetParameters(void) const {return m_params;}
    void Apply(void);

public slots:
    void setDirty(void);

signals:
    void isDirty(void);

private:
    class UILine
    {
    public:
        UILine(std::string &name, QLabel *label, QWidget *widget)
            : m_name(name)
            , m_label(label)
            , m_widget(widget)
        {
        }
        QLabel  *GetLabel(void) {return m_label;}
        QWidget *GetWidget(void) {return m_widget;}
        std::string GetName(void) {return m_name;}
    private:
        std::string m_name;
        QLabel *m_label;
        QWidget *m_widget;
    };

    void AddParameter(std::string name, spectral::Parameters *params);

    spectral::Parameters *m_params;

    std::vector<UILine> m_lines;

    QGridLayout *m_layout;

    bool m_dirty;
};

class StmProcessorDialog : public QDialog
{
    Q_OBJECT
public:
    StmProcessorDialog(QWidget *parent = NULL, spectral::Processor *processor = NULL);

public slots:
    void onOk(void);
    void onCancel(void);
    void onReset(void);
    void onDirty(void);
    
private:
    spectral::Processor *m_processor;
    QVBoxLayout *m_layout;
    QWidget     *m_buttons;
    QHBoxLayout *m_buttonLayout;
    QPushButton *m_ok, *m_cancel, *m_reset;
    StmParameterForm *m_form;
};

#endif
