#ifndef __STM_VIEW_H__
#define __STM_VIEW_H__

#include <vector>

#include <QWidget>
#include <QScrollArea>
#include <QGridLayout>
#include <QDialog>
#include <QLabel>
#include <QComboBox>
#include <QCheckBox>

#include "image.h"
#include "Config.h"


////////////////////////////////////////////////////////////////////////////////
class StmViewWindow : public QWidget
{
    Q_OBJECT
    class GammaObserver : public QtGuiConfigObserver
    {
    public:
        GammaObserver(StmViewWindow *);
        virtual ~GammaObserver()
        {
        }
        virtual void OnSetScreenGamma(float gamma)
        {
            if(m_pane)
            {
                m_pane->setGamma(gamma);
            }
        }

    private:
        StmViewWindow *m_pane;
    };
public:
    StmViewWindow(QWidget *parent = NULL);

    void setGamma(float gamma);
    void SetImage(spectral::Image *);

public slots:
    void onScaleChanged(float scale);
    void onNormaliseChanged(bool normalise);
protected:
    void paintEvent(QPaintEvent *event);

private:
    spectral::Image *CreateRGBImage(spectral::Image *img);
    void CalculateSize(int &, int &);

    float m_gamma;
    spectral::Image *m_image;
    QImage *m_qt_image;
    GammaObserver m_observer;
    bool m_dirty;
    float m_scale;
    bool m_normalise;
};

////////////////////////////////////////////////////////////////////////////////
class StmViewControls : public QWidget
{
    Q_OBJECT
public:
    StmViewControls(QWidget *parent = NULL);
    virtual ~StmViewControls();

public slots:
    void onScaleSelected(int);
    void onNormaliseChanged(int);

signals:
    void scaleChanged(float);
    void normaliseChanged(bool);
private:
    QGridLayout    m_layout;
    QComboBox      m_scaleSelector;
    QComboBox      m_imageSelector;
    QCheckBox      m_normalise;
};

////////////////////////////////////////////////////////////////////////////////
class StmView : public QWidget
{
    Q_OBJECT
public:
    StmView(QWidget *parent = NULL);
    virtual ~StmView();

    virtual QSize sizeHint() const { return QSize(512,512); }
public slots:
    void onImageData(int, void *);
signals:

protected:
private:
    int m_requestedView;
    QGridLayout     m_mainLayout;
    QScrollArea     m_scrollArea;
    StmViewWindow   m_viewWindow;
    StmViewControls m_controlWindow;
};


#endif
