#ifndef __XYVALID_H__
#define __XYVALID_H__

#include <vector>
#include <math.h>

class XyValid
{
    class XyPoint
    {
    public:
        XyPoint(float x, float y)
            : m_x(x)
            , m_y(y)
        {
        }
        float Distance(const XyPoint &other)
        {
            double r2 = ((m_x - other.m_x) * (m_x - other.m_x)) +
                        ((m_y - other.m_y) * (m_y - other.m_y));
            return sqrt(r2);
        }

        float m_x, m_y;
    };
    class XyHalfSpace
    {
    public:
        XyHalfSpace(XyPoint p1, XyPoint p2)
            : m_degenerate(false)
        {
            double direction[2];
            double len;
            m_point[0] = p1.m_x;
            m_point[1] = p1.m_y;
            
            direction[0] = double(p1.m_x)-double(p2.m_x);
            direction[1] = double(p1.m_y)-double(p2.m_y);

            len = sqrt((direction[0] * direction[0]) + (direction[1] * direction[1]));
            direction[0]/= len;
            direction[1]/= len;

            if(len < 1.0e-10)
            {
                m_degenerate = true;
            }

            m_normal[0] = direction[1];
            m_normal[1] = -direction[0];
        };

        bool Contains(float x, float y)
        {
            float dir[2];
            float dp;

            if(m_degenerate) return true; // hack

            dir[0] = x - m_point[0];
            dir[1] = y - m_point[1];

            dp = (dir[0]*m_normal[0]) + (dir[1]*m_normal[1]);
            return (dp < 0.0);
        }
        
        void Invert(void)
        {
            m_normal[0] = -m_normal[0];
            m_normal[1] = -m_normal[1];
        }

        float m_point[2];
        float m_normal[2];
        bool  m_degenerate;
    };
public:
    XyValid();
    bool Valid(float x, float y);
private:
    std::vector<XyHalfSpace> m_edges;
};

#endif
