#include <math.h>

#include "generator.h"

namespace generators {

const float MAX = 1e20;

void 
Generator::DrawLine(spectral::Image *image,
                    void *data,
                    float x1,
                    float y1,
                    float x2,
                    float y2,
                    bool(*draw_pixel)(spectral::Image *, void *, int32_t, int32_t, float, float))
{
    size_t n;
    float length;
    length = sqrt(((x1-x2)*(x1-x2)) +
                   ((y1-y2)*(y1-y2)));
    int32_t xy[2];
    float line_vector[2];
    float line_direction[2];
    float start[2];
    float step[2];
    float dist[2];
    int dir[2];

    start[0] = x1;
    start[1] = y1;

    line_vector[0] = x2 - x1;
    line_vector[1] = y2 - y1;
    
    xy[0] = floor(x1);
    xy[1] = floor(y1);

    // set up.

    for(n=0;n<2;n++)
    {
        // Set up the step directions.
        if(line_vector[n] < 0.0)
        {
            dir[n] = -1;
        }
        else
        {
            dir[n] = 1;
        }
        // We only care about abs values of this from now on.
        line_vector[n] = fabs(line_vector[n]);
        line_direction[n] = line_vector[n] / length;

        // Step sizes in line space.
        if(fabs(line_direction[n]) < 1.0e-6)
        {
            // Parallel with axis
            step[n] = 1.0e20;
            dist[n] = 1.0e20;
        }
        else
        {
            // How long does it take to cross a pixel in this axis?
            step[n] = 1.0 / line_direction[n];
            // how far are we from leaving this pixel?
            {
                float tmp = start[n] * dir[n];
                dist[n] = (1.0 - fmod(tmp, (float)1.0)) / line_direction[n];
            }
        }
    }
    
    // Step along line.
    {
        bool terminate(false);
        float distance(0.0);

        while((distance < length) && !terminate)
        {
            int next;
            float this_step, next_distance;

            //where next?
            if(dist[0] < dist[1])
            {
                next = 0;
            }
            else
            {
                next = 1;
            }
             
            this_step = dist[next];
            next_distance = distance + this_step;
                
            if(draw_pixel)   
            {
                float d1,d2;
                d1 = distance;
                d2 = next_distance;

                if(d2 > length) d2 = length;

                 if(!draw_pixel(image, data, xy[0], xy[1], d1, d2))
                 {
                     terminate = true;
                 }
            }

            // move.
            distance = next_distance;

            for(n=0;n<2;n++)
            {
                dist[n]-= this_step;
            }

            dist[next] = step[next];
            xy[next]+= dir[next];
        }
    }
}

}
