#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "utils.h"
#include "thread.h"
#include "image.h"
#include "parameters.h"

#include "motion_blur.h"

namespace generators {

static bool
DrawPixel(spectral::Image *image, void *p, 
          int x, int y, float start, float end)
{
    bool result(false);
    if(image && p)
    {
        float contrib;
        if((x > 0) && 
           (y > 0) && 
           (uint32_t(x) < image->get_width()) && 
           (uint32_t(y) < image->get_height()))
        {
            float *density = (float *)p;
            {
                contrib = fabs(end - start) * (*density);
            }
            {
                float value = *(image->get_pixel(x, y));
                value+=contrib;
                image->set_pixel(x, y, &value);
            }
            result = true;
        }
    }
    return result;
}

MotionBlur::MotionBlur()
    : Generator()
    , m_length(20)
    , m_angle(0.0)
{
}

MotionBlur::~MotionBlur()
{
}



// Get slots & output type.  Returns output colour space.
// pairs are <slot id, colour space>
uint32_t
MotionBlur::GetInterface(
    std::list<std::pair<uint32_t, uint32_t> > &mandatory,
    std::list<std::pair<uint32_t, uint32_t> > &optional) const
{   
    mandatory.clear();
    optional.clear();
        
    return CLS_LUMINANCE;
}  

bool
MotionBlur::SetImage(uint32_t, const spectral::Image *)
{
    bool result = false;

    return result;
}

void
MotionBlur::ClearImage(uint32_t)
{
}

void 
MotionBlur::GetParameters(spectral::Parameters *params) const 
{
    if(params)
    {
        params->AddParameter(std::string("length"),
                             0.1, 2000.0, m_length);
        params->AddParameter(std::string("angle"),
                             -360.0, 360.0, m_angle);

    }
}

// Apply parameters to a processor.
void
MotionBlur::SetParameters(const spectral::Parameters *params)
{
    if(params)
    {
        params->GetParameter(std::string("length"), m_length);
        params->GetParameter(std::string("angle"), m_angle);
    }
}

spectral::Image *
MotionBlur::Process(void)
{
    spectral::Image *result = NULL;

    uint32_t diameter;
    float radius;

    {
        radius = m_length / 2;
        diameter = floor(radius);
        diameter+=4;
        diameter*= 2;
    }

    result = new spectral::Image(diameter, diameter, CLS_LUMINANCE);

    if(result)
    {
        float centre = (diameter/2)+0.5;
        float density = 1.0/m_length;
        uint32_t x, y;
        for(y=0;y<diameter;y++)
        {
            for(x=0;x<diameter;x++)
            {
                float val(0);
                result->set_pixel(x,y,&val);
            }
        }

        {
            float angle = m_angle;
            angle*=3.14159; // TODO: clean this up.
            angle/=180;
            float x1, y1, x2, y2;
            
            x1 = (float)centre + 0.5 + (cos(angle) * radius);
            y1 = (float)centre + 0.5 + (sin(angle) * radius);
            x2 = (float)centre + 0.5 - (cos(angle) * radius);
            y2 = (float)centre + 0.5 - (sin(angle) * radius);

            DrawLine(result, 
                     &density, 
                     x1, 
                     y1, 
                     x2, 
                     y2,
                     DrawPixel);
        }
    }
    Progress(100);

    return result;
}

}
