#ifndef __RAYS_H__
#define __RAYS_H__

#include <stdint.h>
#include "generator.h"

namespace generators {

class Rays : public Generator
{
public:
    Rays();
    virtual ~Rays();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:

    // data members

    float m_halflife;
    float m_minimum;
    float m_centre_weight;
    float m_angle;
    int32_t m_num_rays;

    int32_t m_samples;
};

}

#endif
