#ifndef __TONEMAP_BUFFER_UTILS_H__
#define __TONEMAP_BUFFER_UTILS_H__

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

#include "utils.h"

#ifdef __cplusplus
extern "C" {
#endif

/*! Convert a CIE xyY buffer to CIE XYZ */
void STM_EXPORT buf_xyy_to_xyz(float *, size_t);

/*! Convert a CIE XYZ buffer to CIE xyY */
void STM_EXPORT buf_xyz_to_xyy(float *, size_t);


/*! Convert a CIE xyY buffer to SRGB without gamma*/
void STM_EXPORT buf_xyy_to_linear_srgb(float *, size_t);

/*! Convert a gamma free SRGB bugger to CIE xyY */
void STM_EXPORT buf_linear_srgb_to_xyy(float *, size_t);

/*! Scale the luminance channel of a CIE xyY buffer */
void STM_EXPORT buf_scale_xyy_y(float *, size_t, float);

/*! Scale all elements of a buffer */
void STM_EXPORT buf_scale(float *, size_t, float);

/*! Convert all elements of a buffer to log values */
void STM_EXPORT buf_luminance_to_density(float *, size_t, float);

/*! Convert all elements of a buffer from log values */
void STM_EXPORT buf_density_to_luminance(float *, size_t);

#ifdef __cplusplus
}
#endif

#endif
