#ifndef __MODULES_H__
#define __MODULES_H__

#include <string>
#include <map>
#include <set>


#ifdef _WIN32
#include <windows.h>
typedef HMODULE MODULE_HANDLE;
#else
typedef void* MODULE_HANDLE;
#endif

#include "stm_export.h"

namespace spectral {

//! Class to manage plugins.
class STM_EXPORT Modules
{
public:
    Modules();
    ~Modules();

    bool LoadModule(std::string, std::string);
    void *CreateObject(const std::string) const;

    void GetLoaders(std::map<std::string, std::string> &) const;
    void GetSavers(std::map<std::string, std::string> &) const;
    void GetProcessors(std::set<std::string> &) const;
    bool GetModuleDesc(const std::string &, std::string &) const;

private:
    void NextExt(char **str);

    class ModuleData
    {
    public:
        MODULE_HANDLE m_module;
        std::string   m_name;
        int           m_type;
        void       *(*m_createFunction)(void);
    };

    std::map<std::string, ModuleData>  m_modules;
    std::map<std::string, std::string> m_fileLoaders;
    std::map<std::string, std::string> m_fileSavers;
    std::set<std::string>              m_processors;
};

}

#endif
