#ifndef __PROCESSOR_H__
#define __PROCESSOR_H__

#include <stdint.h>
#include <list>
#include <utility>

#include "stm_export.h"

namespace control
{
    class ProcessThread;
}

namespace spectral
{
class Image;
class Parameters;

//! The base class for all image processor modules.
class STM_EXPORT Processor
{
public:
    Processor();
    virtual ~Processor();

    Parameters *GetParameters(void) const;

    void Progress(float);

    void SetThread(control::ProcessThread *thread){m_processThread = thread;}

    //! Describe the inputs and outputs of this processor.
    //! Get slots & output type.  Returns output colour space.
    //! pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const = 0;

    //! Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const Image *img) = 0;

    //! Apply parameters to a processor.
    virtual void SetParameters(const Parameters *params) = 0;

    //! Process images.
    virtual Image *Process(void) = 0;

    //! Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);

protected:

    //! Populate a Parameters object.
    virtual void GetParameters(Parameters *params) const = 0;

private:
    control::ProcessThread *m_processThread;
};

}

#endif
