#ifndef __IMAGE_SAMPLER_H__
#define __IMAGE_SAMPLER_H__
#include <stdint.h>
#include "stm_export.h"

namespace spectral
{

class Image;

//! Base class for image sampling utilities.
class STM_EXPORT ImageSampler
{
public:
    enum EdgeMode
    {
        BACKGROUND,
        CLAMP,
        REFLECT,
        WRAP
    };

    ImageSampler(EdgeMode h, EdgeMode v)
        : m_horizontalWrap(h)
        , m_verticalWrap(v)
    {
    }

    virtual ~ImageSampler()
    {
    }

    virtual void SampleImage(const Image *image, 
                             float x, float y,
                             const float *bg,
                             float *result) const = 0;

    void GetEdgeMode(EdgeMode &h, EdgeMode &v) const;

protected:
    const float *GetPixel(const Image *image, float x, float y) const;
    const float *GetPixel(const Image *image, int32_t x, int32_t y) const;
private:
    const EdgeMode m_horizontalWrap, m_verticalWrap;
};

//! Simple implementation of nearest neighbour sampling.
class STM_EXPORT NearestNeighbourSampler : public ImageSampler
{
public:
    NearestNeighbourSampler(EdgeMode h, EdgeMode v)
        : ImageSampler(h, v)
    {
    }

    virtual ~NearestNeighbourSampler()
    {
    }

    virtual void SampleImage(const Image *image, 
                             float x, float y,
                             const float *bg,
                             float *result) const;
};

//! Lancszos image sampler.
class STM_EXPORT LanczosSampler : public ImageSampler
{
public:
    LanczosSampler(EdgeMode h, EdgeMode v, float a)
        : ImageSampler(h, v)
        , m_a(a)
    {
    }

    virtual ~LanczosSampler()
    {
    }

    virtual void SampleImage(const Image *image, 
                             float x, float y,
                             const float *bg,
                             float *result) const;
private:
    const float m_a;
};

}

#endif

