#ifndef __SCHEDULER_H__
#define __SCHEDULER_H__

#include <map>
#include <set>
#include <utility>
#include "stm_export.h"

namespace control {

//! A simple directed graph based job scheduler.
class STM_EXPORT Scheduler
{
public:
    enum JobStatus
    {
        UNKNOWN,
        PENDING,
        STARTED,
        COMPLETE
    };

    void Clear(void);
    void AddNode(int);
    bool AddEdge(int, int);

    void Invalidate(int);
    void SetStarted(int);
    void SetComplete(int);

    void ClearProgress(void);
    void SetProgress(int, float);

    void GetNextScheduledJobs(std::set<int> &) const;

    void GetCompletedJobs(std::set<int> &) const;
    void GetPendingJobs(std::set<int> &) const;
    void GetStartedJobs(std::set<int> &) const;

    float GetProgress(int) const;
    JobStatus GetStatus(int) const;

private:
    std::map<int, std::pair<JobStatus,float> > m_status;
    std::set<std::pair<int, int> >             m_dependencies;
};

}

#endif
