#ifndef __TONEMAP_UTILS_H__
#define __TONEMAP_UTILS_H__

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>

#include "stm_export.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Colour spaces
 */

#define CLS_INVALID          0
#define CLS_SRGB_LINEAR      1
#define CLS_SRGB             2
#define CLS_CIE_XYZ          3
#define CLS_CIE_XYY          4
#define CLS_LOG_LUV          5
#define CLS_LUMINANCE        6
#define CLS_DENSITY          7
#define CLS_NUMBER           8

/* 
 * Types.
 */
typedef float Matrix33[3][3];

uint32_t STM_EXPORT get_pixel_size(uint32_t colour_space);

/*
 * colour utilities.
 */
float STM_EXPORT scalar_product(const float *a, const float *b);
void STM_EXPORT vector_product(const float *a, const float *b, float *out);
void STM_EXPORT vector_normalise(float *a);
void STM_EXPORT transform_vector(const Matrix33, float *vector);

void STM_EXPORT matrix_identity(Matrix33 *m);
void STM_EXPORT matrix_multiply(const Matrix33 m1, 
                                const Matrix33 m2, 
                                Matrix33 *out);
float STM_EXPORT matrix_inverse(const Matrix33 m, Matrix33 *out);


void STM_EXPORT transform_colour(const Matrix33, float *colour);
float STM_EXPORT remove_srgb_gamma(float x);
float STM_EXPORT apply_srgb_gamma(float x);
void STM_EXPORT linear_srgb_to_xyy(float *);
void STM_EXPORT xyy_to_linear_srgb(float *);
void STM_EXPORT xyy_to_xyz(float *);
void STM_EXPORT xyz_to_xyy(float *);
void STM_EXPORT xyz_to_linear_srgb(float *);
void STM_EXPORT chromatic_adaptation(const float *ws, const float *wd, Matrix33 *out);

double STM_EXPORT luminance_to_density(double);
double STM_EXPORT density_to_luminance(double);

/*
 * Misc utils
 */
double STM_EXPORT halton(uint32_t base, uint32_t n);
double STM_EXPORT gaussian(double sigma, double r);

float STM_EXPORT round_float(float x);
double STM_EXPORT round_double(float x);

float STM_EXPORT trunc_float(float x);
double STM_EXPORT trunc_double(float x);

#ifdef __cplusplus
}
#endif

#endif
