#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <string>

#include <ImfArray.h>
#include <ImfRgba.h>
#include <ImfRgbaFile.h>
#include <ImfStandardAttributes.h>

#include "utils.h"
#include "bufutils.h"
#include "exrloader.h"


//
// Main interface
//

// laod an exr image.
spectral::Image *
ExrLoader::LoadImage(const std::string &fn)
{
    spectral::Image *result = NULL;

    int width, height;

    try
    {
        float stonits(1.0);
        Imf::Array2D<Imf::Rgba> pixels;
        Imf::RgbaInputFile file(fn.c_str());
        Imath::Box2i dw = file.dataWindow();
    
        width = dw.max.x - dw.min.x + 1;
        height = dw.max.y - dw.min.y + 1;
    
        if(width && height)
        {
            pixels.resizeErase(height, width);
            file.setFrameBuffer(
                &pixels[0][0]-(dw.min.x+(dw.min.y*width)), 1, width);
            file.readPixels(dw.min.y, dw.max.y);
    
            result = new spectral::Image(width, height, CLS_CIE_XYY);
    
            if(result && result->get_buffer())
            {
                size_t size = size_t(width) * size_t(height);
                float *buf = result->get_buffer();
                float *p = buf;
   
                {
                    // get scale factor.
                    const Imf::FloatAttribute *attr = 
                        file.header().findTypedAttribute<Imf::FloatAttribute>
                            ("whiteLuminance");
                    if(attr)
                    {
                        stonits = attr->value();
                    }
                    std::cerr << "stonits = " << stonits << std::endl;
}

                for(int y=0;y<height;y++)
                {
                    for(int x=0;x<width;x++)
                    {
                        float r, g, b;
                        r = float(pixels[y][x].r);
                        g = float(pixels[y][x].g);
                        b = float(pixels[y][x].b);

                        *p++ = (r>=0.0)?r:0.0;
                        *p++ = (g>=0.0)?g:0.0;
                        *p++ = (b>=0.0)?b:0.0;
                    }
                }
    
                buf_linear_srgb_to_xyy(buf, size);
                buf_scale(buf, size, stonits);
            }
        }

    }
    catch(...)
    {
        if(result)
        {
            delete result;
            result = NULL;
        }
    }
    
    return result;
   
}    
