#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <string>

#include "utils.h"
#include "bufutils.h"
#include "htond.h"
#include "pfmloader.h"


//
// Main interface
//

float swap_bytes(float in)
{
    float result;
    unsigned char *p1, *p2;

    p1 = (unsigned char *)(&in);
    p2 = (unsigned char *)(&result);

    p2[0]=p1[3];
    p2[1]=p1[2];
    p2[2]=p1[1];
    p2[3]=p1[0];

    return result;
}

// laod an exr image.
spectral::Image *
PfmLoader::LoadImage(const std::string &fn)
{
    spectral::Image *result = NULL;

    int width, height;
    FILE *fp;

    fp = fopen(fn.c_str(), "rb");

    if(fp)
    {
        bool little_endian(false);
        bool colour(false);
        bool valid(true);
        float stonits(1.0);
        char line[256];
        size_t size, buf_size;

        // read header.
        if(fgets(line, 255, fp))
        {
            if(strncmp(line, "PF", 2) == 0)
            {
                colour = true;
            }
            else if(strncmp(line, "Pf", 2) == 0)
            {
                colour = false;
            }
            else
            {
                valid = false;
            }
        }
        else
        {
            valid = false;
        }

        // dimensions.
        if(valid)
        {
            if(fgets(line, 255, fp))
            {
                if(sscanf(line, "%i %i", &width, &height) == 2)
                {
                    if(colour)
                    {
                         result = new spectral::Image(width, height, CLS_CIE_XYY);
                    }
                    else
                    {
                         result = new spectral::Image(width, height, CLS_LUMINANCE);
                    }
                }
                else
                {
                    valid = false;
                }
            }
            else
            {
                valid = false;
            }
        }

        // scale/endian
        if(valid)
        {
            if(fgets(line, 255, fp))
            {
                if(sscanf(line, "%f", &stonits) == 1)
                {
                    little_endian = (stonits < 0);
                    stonits = fabs(stonits);
                }
                else
                {
                    delete result;
                    result = NULL;
                    valid = false;
                }
            }
            else
            {
                delete result;
                result = NULL;
                valid = false;
            }    
        }

        // read pixels
        if(valid)
        {
            size = size_t(width) * size_t(height);
            buf_size = size;

            if(colour)
            {
                buf_size*= 3;
            }

            if(fread(result->get_buffer(), sizeof(float), buf_size, fp) != buf_size)
            {
                delete result;
                result = NULL;
                valid = false;
            }
        }

        // process pixels
        if(valid)
        {
            size_t i;
            float *buf = result->get_buffer();

            if(little_endian)
            {
                // make big endian for sp_ntohf
                for(i=0;i<buf_size;i++)
                {
                    buf[i] = swap_bytes(buf[i]);
                }
            }

            // convert to native byteorder and apply scale
            for(i=0;i<buf_size;i++)
            {
                buf[i] = sp_ntohf(buf[i]);
            }

            // convert
            if(colour)
            {
                buf_linear_srgb_to_xyy(buf, size);
            }

            // scale
            if(stonits != 1.0)
            {
                if(colour)
                {
                    buf_scale_xyy_y(buf, stonits, size);
                }
                else
                for(i=0;i<buf_size;i++)
                {
                    buf[i]*= stonits;
                }
            }

            // flip vertical
            {
                size_t mid = (height/2)+1;
                size_t y1, y2;
                float *tmp;
                size_t line_size;

                line_size = width;
                if(colour)
                {
                    line_size*= 3;
                }
                tmp = (float *)calloc(line_size, sizeof(float));

                for(y1=0;y1<mid;y1++)
                {
                    y2 = height - y1 - 1;
                    if(y1 < y2)
                    {
                        float *p1, *p2;

                        p1 = buf + (line_size * y1);
                        p2 = buf + (line_size * y2);

                        memcpy(tmp, p1, line_size * sizeof(float));
                        memcpy(p1, p2, line_size * sizeof(float));
                        memcpy(p2, tmp, line_size * sizeof(float));
                    }
                }
                free(tmp);
            }
        }
        
        fclose(fp);
    }
    return result;
   
}    
