#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <string>

#include <png.h>
#include <htond.h>

#include "utils.h"
#include "pngloader.h"
#include "bufutils.h"


//
// Main interface
//

// laod an png image.
spectral::Image *
PngLoader::LoadImage(const std::string &fn)
{
    spectral::Image *result = NULL;


    if(fn.length())
    {
        float stonits(1.0);
        FILE *fp;
        int depth, coltype, channels;
        unsigned long rowbytes;

        png_structp png(NULL);
        png_infop info(NULL);

        if((fp = fopen(fn.c_str(), "rb")) != NULL)
        {
            bool success(true);
            int width(0), height(0);
            unsigned char *image(NULL);

            // Check we have a png.
            {
                unsigned char hdr[8];
                
                if(fread(hdr, 1, 8, fp) != 8)
                {
                    success = false;
                }

                if(!png_check_sig(hdr, 8))
                {
                    success = false;
                }
            }

            if(success)
            {
                png = png_create_read_struct(PNG_LIBPNG_VER_STRING,
                                             NULL, NULL, NULL);
                if(png)
                {
                    info = png_create_info_struct(png);
                    if(!info)
                    {
                        success = false;
                    }
                }
                else
                {
                    success = false;
                }
            }

            if(success)
            {
                // I hate setjmp/longjmp!
                if(setjmp(png_jmpbuf(png)))
                {
                    success = false;
                }
                else
                {
                    // setup png import options
                    png_uint_32 w, h;
                    double gamma(1.0);

                    png_init_io(png, fp);
                    png_set_sig_bytes(png, 8);
                    png_read_info(png, info);
                    png_get_IHDR(png, info, &w, &h, &depth, &coltype,
                                 NULL, NULL, NULL);
                    
                    width = int(w);
                    height = int(h);
                    
                    // Set up various flags.
                    switch(coltype)
                    {
                    case PNG_COLOR_TYPE_PALETTE:
                        png_set_expand(png);
                        break;
                    case PNG_COLOR_TYPE_GRAY:
                    case PNG_COLOR_TYPE_GRAY_ALPHA:
                        png_set_gray_to_rgb(png);
                        break;
                    }

                    if(depth < 8)
                    {
                        png_set_expand(png);
                        depth = 8;
                    }
                    if(png_get_valid(png, info, PNG_INFO_tRNS))
                    {
                        png_set_expand(png);
                        png_set_strip_alpha(png);
                    }

                    if(coltype & PNG_COLOR_MASK_ALPHA)
                    {
                        png_set_strip_alpha(png);
                    }

                    if(png_get_gAMA(png, info, &gamma))
                    {
                        // we want this linearised!
                        m_gamma = gamma;
                    }
                    else
                    {
                        m_gamma = 1.0/2.2;
                    }
                    printf("gamma = %f\n", m_gamma);

                    png_read_update_info(png, info);
                }
            }

            if(success)
            {
                // Read the image.
                rowbytes = png_get_rowbytes(png, info);
                channels = png_get_channels(png, info);
               
                if(!channels)
                {
                    // TODO
                }                

                image = (unsigned char *)malloc(rowbytes * height);

                if(image)
                {
                    png_bytepp  rows(NULL);
                    rows = (png_bytepp)malloc(height * sizeof(png_bytep));

                    if(rows)
                    {
                        for(size_t row=0;row<size_t(height);row++)
                        {
                            rows[row]= image + (row * rowbytes);
                        }
                        png_read_image(png, rows);
                        free(rows);
                        png_read_end(png, NULL);
                    }
                    else
                    {
                        success = false;
                    }
                }
                else
                {
                    success = false;
                }
            }

            if(success)
            {
                // Convert our image into a floating point buffer.
                result = new spectral::Image(width, height, CLS_CIE_XYY);
                
                if(result)
                {
                    size_t size = size_t(width) * size_t(height);
                    float *buf = result->get_buffer();

                    if(depth == 8)
                    {
                        unsigned char *p = (unsigned char *)image;
                        float *flp = buf;

                        for(size_t n=0;n<size*3;n++)
                        {
                            float tmp;
                            tmp = *p;
                            tmp/= 255.0;
                            tmp = pow(tmp, 1.0/m_gamma);
                            *flp = tmp;

                            p++;
                            flp++;
                        }
                    }
                    else if(depth == 16)
                    {
                        unsigned short *p = (unsigned short *)image;
                        float *flp = buf;

                        for(size_t n=0;n<size*3;n++)
                        {
                            float tmp;
                            tmp = sp_ntohs(*p);
                            tmp/= 65535.0;
                            tmp = pow(tmp, 1.0/m_gamma);
                            *flp = tmp;

                            p++;
                            flp++;
                        }
                    }
                    else
                    {
                        success = false;
                    }
                    if(success)
                    {
                        buf_linear_srgb_to_xyy(buf, size);
                        buf_scale_xyy_y(buf, size, stonits);
                    }
                }
                else
                {
                    success = false;
                }

            }

            if(png)
            {
                 png_destroy_read_struct(&png, NULL, NULL);
            }
            if(info)
            {
                //png_destroy_info_struct(info);
            }
            if(image)
            {
                free(image);
            }
            if(!success)
            {
                if(result)
                {
                    delete result;
                    result = NULL;
                }
            }

            fclose(fp);
        }
    }
    
    return result;
   
}    
