#ifndef __SPECTRAL_ALLOC_H__
#define __SPECTRAL_ALLOC_H__

#include <stddef.h>
#include <vector>

namespace spectral {

template <typename T>
class simple_allocator
{
    typedef struct _wrapper
    {
        struct _wrapper *m_next;
        T m_object;
    } wrapper;
public:
    simple_allocator<T>(size_t count = 128)
        : m_count(count)
        , m_first_free(NULL)
    {
        if(m_count < 1)
        {
            m_count = 64;
        }
    }

    ~simple_allocator<T>()
    {
        for(size_t i=0;i<m_buffer.size();i++)
        {
            if(m_buffer[i])
            {
                delete[] m_buffer[i];
            }
        }
    }

    void extend_pool(void)
    {
        wrapper *new_buf = new wrapper[m_count];

        if(new_buf)
        {
            for(size_t i=0;i<(m_count - 1);i++)
            {
                new_buf[i].m_next = &(new_buf[i+1]);
            }
            new_buf[m_count-1].m_next = m_first_free;
            m_first_free = new_buf;
            m_buffer.push_back(new_buf);
        }
    }

    T *alloc(void)
    {
        T *new_obj(NULL);
        if(m_first_free)
        {
            new_obj = &(m_first_free->m_object);
            m_first_free = m_first_free->m_next;
        }
        else
        {
            extend_pool();
            new_obj = alloc();
        }
        return new_obj;
    }

    void free(T *to_go)
    {
        if(to_go)
        {
            size_t offset = offsetof(_wrapper, m_object);
            char *p = (char *)to_go;
            p-= offset;
            wrapper *to_go_wrapper = (wrapper *)p;
            to_go_wrapper->m_next = m_first_free;
            m_first_free = to_go_wrapper;
        }
    }

private:
    size_t m_count;
    std::vector<wrapper*> m_buffer;
    wrapper *m_first_free;
};

}

#endif

