#ifndef __AVL_TREE_H__
#define __AVL_TREE_H__

#include <stdlib.h>
#include <stdint.h>
namespace bilateral
{

class Node
{
public:

    Node(int64_t value, int64_t squared, int64_t cubed);
    ~Node();

    // Data update.

    bool Insert(Node *node, Node **root);
    bool Delete(Node *to_go, Node **root);
    bool Remove(Node **root);
    Node *Find(int64_t);

    void GetTotals(int64_t min, int64_t max, 
                   int64_t &total_values,
                   int64_t &total_squares,
                   int64_t &total_cubes,
                   uint32_t &total_count);

    void UpdateTotals(bool); 

    static bool RotateLeft(Node *);
    static bool RotateRight(Node *);
    static bool DoubleRotateLeft(Node *);
    static bool DoubleRotateRight(Node *);

    // Data access
    Node *GetLeft(void) const;
    Node *GetRight(void) const;

    int64_t GetValue(void) const;
    int64_t GetSquareValue(void) const;
    int64_t GetCubeValue(void) const;
    uint32_t GetCount(void) const;
    void SetCount(uint32_t);

    int64_t GetTotalValue(void) const;
    int64_t GetTotalSquareValue(void) const;
    int64_t GetTotalCubeValue(void) const;
    uint32_t GetTotalCount(void) const;

// debugging functions.
    void Dump(void);
    void Dump(int);
    bool Check(void);

protected:

    int32_t GetBalance(void) const;
    void SetBalance(int32_t);

    void ExcludeMin(int64_t min, 
                    int64_t &total_values,
                    int64_t &total_squares,
                    int64_t &total_cubes,
                    uint32_t &total_count);

    void ExcludeMax(int64_t min, 
                    int64_t &total_values,
                    int64_t &total_squares,
                    int64_t &total_cubes,
                    uint32_t &total_count);

    void SetLeft(Node *);
    void SetRight(Node *);
    void SetParent(Node *);

    Node *GetParent(void) const;

    Node *FindRoot(void);
    Node *FindMax(void);
    Node *FindMin(void);

    bool Rebalance(Node **root);
private:
    Node *m_left, *m_right, *m_parent;
    int32_t m_balance;

    int64_t m_value, m_square_value, m_cube_value;
    int64_t m_total_value, m_total_square_value, m_total_cube_value;
    uint32_t m_total_count;
    uint32_t m_count;
};

}
#endif

