#ifndef __AVL_TREE_2_H__
#define __AVL_TREE_2_H__

#include <stdlib.h>
#include <stdint.h>
#include "alloc.h"

namespace bilateral
{

typedef struct _avl_node
{
    struct _avl_node *m_left, *m_right, *m_parent;

    int64_t m_value, m_square_value, m_cube_value;
    int64_t m_total_value, m_total_square_value, m_total_cube_value;
    uint32_t m_total_count;
    uint32_t m_count;
    int32_t m_balance;
} avl_node;

typedef spectral::simple_allocator<avl_node> allocator;

// public stuff
avl_node *
create_node(allocator &alloc,
            int64_t value, 
            int64_t squared,
            int64_t cubed);

void
destroy_node(allocator &alloc,
             avl_node *to_go);

bool insert_node(avl_node *node, avl_node *to_insert, avl_node **root);
bool delete_node(allocator &alloc, avl_node *node, avl_node *to_go, avl_node **root);
bool remove_node(allocator &alloc, avl_node *node, avl_node **root);
void update_totals(avl_node *node, bool propagate);

avl_node *find_node(avl_node *node, int64_t key);

void get_totals(avl_node *,
                int64_t min, int64_t max,
                int64_t &total_values,
                int64_t &total_squares,
                int64_t &total_cubes,
                uint32_t &total_count);

}
#endif

