#ifndef __BILATERAL_H__
#define __BILATERAL_H__

#include <vector>

#include <stdint.h>

#include "utils.h"
#include "image.h"
#include "processor.h"
#include "thread.h"

namespace bilateral
{

////////////////////////////////////////////////////////////////////////////////

//! A processor to implement Bilateral filtering.
class Bilateral : public spectral::Processor
{
public:
    Bilateral();
    ~Bilateral();

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);

protected:

    // Populate a Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    //! Internal image processing function.
    void CrunchImage(const spectral::Image *reference,
                     const spectral::Image *input,
                     int32_t radius,
                     bool wrap_horizontal,
                     bool wrap_vertical,
                     uint32_t pass,
                     uint32_t num_passes,
                     spectral::Image *result);
                     

    const spectral::Image *m_lumImage;
    int32_t m_radius;
    int32_t m_passes;
    float m_threshold;
    bool  m_wrap_horizontal, m_wrap_vertical;
};

}

#endif
