#ifndef __HISTOGRAM_H__
#define __HISTOGRAM_H__

#include <stdio.h>
#include <stdint.h>
#include <math.h>

#define USE_ALLOCATOR

#ifdef USE_ALLOCATOR
#include "alloc.h"
#include "avltree2.h"
#else
#include "avltree.h"
#endif

namespace bilateral
{

class SubHistogram 
{
public:
    SubHistogram();
    virtual ~SubHistogram();

    bool AddValue(int64_t val, int64_t square, int64_t cube);
    bool RemoveValue(int64_t val);
    void Clear(void);

    void GetSums(int64_t min, int64_t max, 
                 int64_t &sum, 
                 int64_t &sum_squares,
                 int64_t &sum_cubes,
                 uint32_t &count) const;
#ifdef USE_ALLOCATOR
    void SetAllocator(allocator *);
#endif
private:
#ifdef USE_ALLOCATOR
    allocator *m_allocator;
    avl_node *m_tree;
#else
    Node *m_tree;
#endif
};


const uint32_t NUM_LAYERS = 5;

class BoxHistogram
{
public:
    BoxHistogram(uint32_t width, uint32_t diameter);
    ~BoxHistogram(); 

    bool AddValue(uint32_t x, float val);
    bool RemoveValue(uint32_t x, float val);
    void Clear(void);

    void GetSums(uint32_t xmin, uint32_t xmax,
                 float min, float max, 
                 float &sum, 
                 float &sum_squares,
                 float &sum_cubes,
                 uint32_t &count) const;
private:
    uint32_t GetLayerSize(uint32_t layer) const;

    SubHistogram *m_layers[NUM_LAYERS];
    uint32_t m_count[NUM_LAYERS];
    uint32_t m_masks[NUM_LAYERS];
    uint32_t m_width;
    uint32_t m_use_layers;
#ifdef USE_ALLOCATOR
    allocator m_allocator;
#endif

};

}
#endif

