#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//#include <assert.h>

#include <vector>
#include <set>

#include "parameters.h"
#include "image.h"
#include "utils.h"
#include "bufutils.h"
#include "thread.h"

#include "mlbilateral.h"

#include "filter.h"

#define IMG_LUMINANCE 0

namespace spectral {

////////////////////////////////////////////////////////////////////////////////
//
// MlBilateral processor implementation.
//
////////////////////////////////////////////////////////////////////////////////

MlBilateral::MlBilateral()
    : Processor()
    , m_lumImage(NULL)
    , m_baseLayer(NULL)
    , m_large_threshold(0.1)
    , m_small_threshold(0.3)
    , m_large_contrast(1.0)
    , m_medium_contrast(1.0)
    , m_small_contrast(1.0)
    , m_base_radius(2)
    , m_num_layers(5)
    , m_wrap_horizontal(false)
    , m_wrap_vertical(false)
{
}

MlBilateral::~MlBilateral()
{
}

// Get slots & output type.  Returns output colour space.
// pairs are <slot id, colour space>
uint32_t
MlBilateral::GetInterface(
    std::list<std::pair<uint32_t, uint32_t> > &mandatory,
    std::list<std::pair<uint32_t, uint32_t> > &optional) const
{
    mandatory.clear();
    optional.clear();

    mandatory.push_back(
        std::pair<uint32_t, uint32_t>(IMG_LUMINANCE, CLS_LUMINANCE));

    return CLS_LUMINANCE;
}


bool
MlBilateral::SetImage(uint32_t slot, const Image *img)
{
    if(slot == IMG_LUMINANCE)
    {
        m_lumImage = NULL;

        if(img)
        {
            if(img->get_colour_space() == CLS_LUMINANCE)
            {
                m_lumImage = img;

                return true;
            }
        }
        else
        {
            return true;
        }
    }
    return false;
}

void
MlBilateral::GetParameters(Parameters *params) const
{
    if(params)
    {
        std::string tmp;
/*
        tmp = "radius";
        params->AddParameter(tmp, 1, 100, m_radius);
        
        tmp = "threshold";
        params->AddParameter(tmp, 0.1, 10.0, m_threshold);

        tmp = "passes";
        params->AddParameter(tmp, 1, 10, m_passes);
        
        tmp = "wrap horizontal";
        params->AddParameter(tmp, m_wrap_horizontal);

        tmp = "wrap vertical";
        params->AddParameter(tmp, m_wrap_vertical);
*/
    }
}

void
MlBilateral::SetParameters(const Parameters *params)
{
    if(params)
    {
       std::string tmp;
/*
       tmp = "radius";
       params->GetParameter(tmp, m_radius);
       
       tmp = "threshold";
       params->GetParameter(tmp, m_threshold);
        
        tmp = "passes";
        params->GetParameter(tmp, m_passes);
        
        tmp = "wrap horizontal";
        params->GetParameter(tmp, m_wrap_horizontal);

        tmp = "wrap vertical";
        params->GetParameter(tmp, m_wrap_vertical);
*/
    }
}

Image *
MlBilateral::Process(void)
{
    Image *result(NULL);

    Progress(0.0);

    if(m_lumImage)
    {
        if(0)
        {
            CleanUp();
        }

        if((m_baseLayer == NULL) || (m_layers.size() == 0))
        {
            CleanUp();
            CreateLayers();
        }

        result = Generate();
    }
    else
    {
        CleanUp();
    }

    Progress(100.0);

    return result;
}

/*
        int32_t i(m_passes);
        Image *reference(NULL);
        Image *input(NULL);
        float scale = m_threshold/logf(2.0);        
  

        reference = new spectral::Image(*m_lumImage);
        buf_luminance_to_density(reference->get_buffer(), 
                                 reference->get_width() * 
                                 reference->get_height(),
                                 1.0e-10);

        buf_scale(reference->get_buffer(), 
                  reference->get_width() * reference->get_height(),
                  1.0 / scale);

        result = new spectral::Image(*reference);
        input = new spectral::Image(*reference);
 
        do
        {
            // Create a new output image.
            CrunchImage(reference, input, m_radius,
                        m_wrap_horizontal, m_wrap_vertical,
                        m_passes - i,
                        m_passes,
                        result);
            

            i--;

            if(i)
            {
                // We have another pass to go.  Swap input & result.
                Image *tmp;
                tmp = input;
                input = result;
                result = tmp;
            }
        } while(i); 

        // Clean up.
        delete input;
        delete reference;

        // Rescale.
        buf_scale(result->get_buffer(), 
                  result->get_width() * result->get_height(),
                  scale);

        buf_density_to_luminance(result->get_buffer(), 
                                 result->get_width() * result->get_height());
*/

void
MlBilateral::ClearImage(uint32_t slot)
{
    if(slot == IMG_LUMINANCE)
    {
        m_lumImage = NULL;
    }
}

void
MlBilateral::CrunchImage(const Image *reference,
                       const Image *input,
                       int32_t radius,
                       bool wrap_horizontal,
                       bool wrap_vertical,
                       uint32_t pass,
                       uint32_t num_passes,
                       Image *result)
{
    float min_progress(0);
    float max_progress(100.0);

    if(num_passes)
    {
        min_progress = ((float)pass / (float)num_passes);
        max_progress = ((float)(pass + 1) / (float)num_passes);
        min_progress*= 95.0; 
        max_progress*= 95.0; 
    }

    Filter(this, min_progress, max_progress, reference, input, radius,
           wrap_horizontal, wrap_vertical, result);
}
      
MlBilateral::CleanUp(void)
{
    if(m_baseLayer)
    {
        delete m_baseLayer;
        m_baseLayer = NULL;
    }

    if(m_layers.size())
    {
        for(size_t i=0;i<m_layers.size();i++)
        {
            if(m_layers[i])
            {
                delete m_layers[i];
            }
        }
        m_layers.clear();
    }
}

}
