#include "utils.h"
#include "image.h"
#include "parameters.h"

#include "coltolum.h"

const uint32_t IMG_PICTURE = 0;

namespace coltolum {

ColourToLuminance::ColourToLuminance()
    : spectral::Processor()
    , m_picture(NULL)
{
}

ColourToLuminance::~ColourToLuminance()
{
}

// Get slots & output type.  Returns output colour space.
// pairs are <slot id, colour space>
uint32_t
ColourToLuminance::GetInterface(
    std::list<std::pair<uint32_t, uint32_t> > &mandatory,
    std::list<std::pair<uint32_t, uint32_t> > &optional) const
{   
    mandatory.clear();
    optional.clear();
        
    mandatory.push_back(     
        std::pair<uint32_t, uint32_t>(IMG_PICTURE, CLS_CIE_XYY));
            
    return CLS_LUMINANCE;
}  

bool
ColourToLuminance::SetImage(uint32_t type, const spectral::Image *img)
{
    bool result = false;

    if(type == IMG_PICTURE)
    {
        m_picture = NULL;

        if(img)
        {
            if(img->get_colour_space() == CLS_CIE_XYY)
            {
                m_picture = img;
                result = true;
            }
        }
        else
        {
            result = true;
        }
    }

    return result;
}

void
ColourToLuminance::ClearImage(uint32_t type)
{
    if(type == IMG_PICTURE)
    {
        if(m_picture)
        {
            m_picture = NULL;
        }
    }
}

void 
ColourToLuminance::GetParameters(spectral::Parameters *) const 
{
}

// Apply parameters to a processor.
void
ColourToLuminance::SetParameters(const spectral::Parameters *)
{
}

spectral::Image *
ColourToLuminance::Process(void)
{
    spectral::Image *result = NULL;

    Progress(0);
    if(m_picture)
    {
        
        result = new spectral::Image(m_picture->get_width(), 
                                     m_picture->get_height(),
                                     CLS_LUMINANCE);
    }

    if(result)
    {
        size_t in_idx, out_idx;
        size_t size = size_t(result->get_width()) * 
                      size_t(result->get_height());
        float *out_buf = result->get_buffer();
        float *in_buf = m_picture->get_buffer();

        in_idx = 2;

        for(out_idx=0,in_idx=2;out_idx<size;out_idx++,in_idx+=3)
        {
            if((out_idx % 262144) == 0)
            {
                float progress;
                progress = out_idx;
                progress/= size;
                progress*= 100;
                Progress(progress);
            }
            out_buf[out_idx] = in_buf[in_idx];
        }

        Progress(100);
    }

    return result;
}

}
