#ifndef __CONTRAST_H__
#define __CONTRAST_H__

#include <stdint.h>
#include "processor.h"

namespace contrast {

//! Log space contrast enhancement of an xyY colour image using a
//! local adaptation luminance image.
class Contrast : public spectral::Processor
{
public:
    Contrast();
    virtual ~Contrast();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:

    // data members
    const spectral::Image *m_picture;
    const spectral::Image *m_lal;

    float m_contrast;
    bool  m_flatten;
};

}

#endif
