#ifndef __PHOTOGRAPHIC_H__
#define __PHOTOGRAPHIC_H__

#include <stdint.h>
#include "processor.h"

namespace convolution {

//! Convolve a colour image with a luminance channel.
class ConvolveMono : public spectral::Processor
{
public:
    ConvolveMono();
    virtual ~ConvolveMono();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

    void GetFullSize(uint32_t &width, uint32_t &height, uint32_t &offset_x, uint32_t &offset_y);
    void *CreateImageBuffer(spectral::Image *, uint32_t n);
    void UnpackImageBuffer(void *,spectral::Image *, uint32_t n);
    void *CreateFilterBuffer(uint32_t n);

private:
    // data members
    const spectral::Image *m_picture;
    const spectral::Image *m_filter;

    bool m_wrap_horizontal, m_wrap_vertical;
};

}

#endif
