#ifndef __FIELD_H__
#define __FIELD_H__

#include <stdint.h>
#include "processor.h"
#include "image.h"

namespace spectral {

struct weights
{
    int   index[4];
    float weight[4];
};

class Field : public Processor
{
public:
    Field();
    virtual ~Field();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const Parameters *);

    // Process images.
    virtual Image *Process(void) = 0;

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a Parameters object.
    virtual void GetParameters(Parameters *) const;

protected:
    spectral::image<float> *MakeField(void);

    void GetWeight(int32_t width, int32_t height, float x, float y, struct weights &);

    void BuildStreamline(float x, float y, 
                         spectral::image<float> *field, 
                         std::list<std::pair<float, float> > &result);

    void FollowStreamline(std::list<std::pair<float, float> >&line,
                          spectral::Image *src,
                          spectral::Image *result,
                          spectral::image<float> *hits); 

    void DarkStreamline(std::list<std::pair<float, float> >&line,
                          spectral::Image *result,
                          spectral::image<float> *hits); 

    bool GetFieldDirection(spectral::image<float> *field, 
                           float *pos, float *dir);
    bool GetColour(const spectral::Image *, float x, float y, float *colour);

    bool SplatColour(float x, float y, float *colour, 
                     spectral::Image *, spectral::image<float> *, float weight = 1);
    // data members
    const spectral::Image *m_picture;
    const spectral::Image *m_field;

    float m_weight;
    int m_size;
    bool m_fine;
    bool m_contour;
    int32_t m_streamline_length;
};

}

#endif
