#include <stdlib.h>
#include <math.h>

#include <stdio.h>
#include <iostream>
#include <list>
#include <utility>

#include "utils.h"
#include "image.h"
#include "parameters.h"

#include "flic.h"

namespace spectral {

Flic::Flic()
    : Field()
    , m_threshold(0.5)
{
}

Flic::~Flic()
{
}

void 
Flic::GetParameters(Parameters *params) const 
{
    Field::GetParameters(params);

    if(params)
    {
/*
    , m_threshold(.3)
    , m_size(30)
    , m_fine(true)
    , m_contour(true)
*/
    }
}

// Apply parameters to a processor.
void
Flic::SetParameters(const Parameters *params)
{
    Field::SetParameters(params);

    if(params)
    {
    }
}

spectral::Image *
Flic::Process(void)
{
    spectral::Image *result = NULL;
    spectral::Image *src = NULL;
    spectral::image<float> *hits = NULL;

    Progress(0);

    if(m_picture && m_field && 
      (m_picture->get_width() == m_field->get_width()) &&
      (m_picture->get_height() == m_field->get_height()))
    {
        result = new spectral::Image(*m_picture);
        src = new spectral::Image(*m_picture);
        hits = new spectral::image<float>(m_picture->get_width(),
                                          m_picture->get_height(), 1);
    }

    if(result && src && hits)
    {
        int i = 0;
        int ignored = 0;
        int max_ignored = 0;
        int width = m_picture->get_width();
        int height = m_picture->get_height();

        float *hits_buf = hits->get_buffer();

        //std::cerr << "Making vector field" << std::endl;
        spectral::image<float> *field = MakeField();

        //std::cerr << "Initialise weights" << std::endl;
        for(int iy=0;iy<height;iy++)
        {
            for(int ix=0;ix<width;ix++)
            {
                float pixel[3];
                float val;
                val = 0.1;
                {
                    const float *tmp;
                    tmp = m_picture->get_pixel(ix, iy);
                    pixel[0] = tmp[0];
                    pixel[1] = tmp[1];
                    pixel[2] = tmp[2];
                }
                xyy_to_xyz(pixel);

                src->set_pixel(ix, iy, pixel);
		pixel[0]*=val;
		pixel[1]*=val;
		pixel[2]*=val;
                result->set_pixel(ix, iy, pixel);
                hits->set_pixel(ix, iy, &val);
            }
        }
        i = 1;
        while(ignored < 300)
        {
            float x, y;
            struct weights filter;
            bool skip;

            x = halton(5, i);
            y = halton(3, i);
            x*= width;
            y*= height;

            i++;

            GetWeight(width, height, x, y, filter);

            {
                skip = true;

                for(int j=0;j<4;j++)
                {
                    if(filter.index[j] != -1)
                    {
                        if(hits_buf[filter.index[j]] < m_threshold)
                        {
                            skip = false;
                        }
                    }
                }

                if(skip)
                {
                    // We don't need to work on this pixel.
                    ignored++;
                    if(ignored > max_ignored)
                    {
                        max_ignored = ignored;
                        std::cerr << "ignored " << ignored << std::endl;
                    }
                }
                else
                {
                    // Create a streamline, starting here, and convolve along
                    // it while reseting ignore count.
                    ignored = 0;
                    
                    std::list<std::pair<float, float> > line;
                    line.clear();

                    BuildStreamline(x, y, field, line);

                    if(line.size())
                    {
                        FollowStreamline(line, src, result, hits);
                    }
                    else
                    {
                    /*
                        float colour[3];
                        GetColour(src, 
                                  x,
                                  y,
                                  colour);
                        SplatColour(x, y, colour, result, hits);
                    */
                    }
                }
            }
        }

        std::cerr << "mop up 1" << std::endl;
        for(int iy = 0;iy<height; iy++)
        {
            float y;
            y = iy;
            y+= 0.5;
            for(int ix = 0;ix<width; ix++)
            {
                float x;
                x = ix;
                x+= 0.5;
    
                if(*(hits->get_pixel(ix,iy)) < m_threshold)
                {
                    std::list<std::pair<float, float> > line;
    
                    BuildStreamline(x, y, field, line);
                    //std::cerr << "built line " << line.size() << std::endl;
                    if(line.size())
                    {
                        FollowStreamline(line, src, result, hits);
                    }
                }
            }
        }

        std::cerr << "mop up 2" << std::endl;

        for(int iy = 0;iy<height; iy++)
        {
            float y;
            y = iy;
            y+= 0.5;
            for(int ix = 0;ix<width; ix++)
            {
                float x;
                x = ix;
                x+= 0.5;
    
                if(*(hits->get_pixel(ix,iy)) < m_threshold)
                {
                    std::list<std::pair<float, float> > line;
    
                    {
                        float val;
                        float total[3];
                        total[0]=total[1]=total[2]=0.0;
                        val = 0.0;

                        if(x > 0)
                        {
                            val+=1;
                            const float *colour = src->get_pixel(ix-1, iy);
                            total[0]+=colour[0];
                            total[1]+=colour[1];
                            total[2]+=colour[2];
                        }
                        if(y > 0)
                        {
                            val+=1;
                            const float *colour = src->get_pixel(ix, iy-1);
                            total[0]+=colour[0];
                            total[1]+=colour[1];
                            total[2]+=colour[2];
                        }
                        if(x < (width-1))
                        {
                            val+=1;
                            const float *colour = src->get_pixel(ix+1, iy);
                            total[0]+=colour[0];
                            total[1]+=colour[1];
                            total[2]+=colour[2];
                        }
                        if(y > (height-1))
                        {
                            val+= *(hits->get_pixel(ix+1, iy));
                            const float *colour = src->get_pixel(ix, iy+1);
                            total[0]+=colour[0];
                            total[1]+=colour[1];
                            total[2]+=colour[2];
                        }
                        result->set_pixel(ix, iy, total);
                        hits->set_pixel(ix, iy, &val);
                        //SplatColour(x, y, colour, result, hits);
                    }
                }
            }
        }

        for(int iy=0;iy<height;iy++)
        {
            for(int ix=0;ix<width;ix++)
            {
                float pixel[3];
                const float *tmp;

                tmp = result->get_pixel(ix, iy);

                pixel[0] = tmp[0];
                pixel[1] = tmp[1];
                pixel[2] = tmp[2];

                xyz_to_xyy(pixel);

                tmp = hits->get_pixel(ix, iy);

                if(*tmp > 1.0e-6)
                    pixel[2]/= *tmp;
                else
                    std::cerr << "Error" << ix << ", " << iy << "\n";

                result->set_pixel(ix, iy, pixel);
            }
        }
        delete src;
        delete field;
        delete hits;
    }

    Progress(100);
    
    return result;
}

}
