#include <stdlib.h>
#include <math.h>

#include <stdio.h>
#include <iostream>
#include <list>
#include <utility>

#include "utils.h"
#include "image.h"
#include "parameters.h"

#include "lines.h"

namespace spectral {

Lines::Lines()
    : Field()
    , m_number_of_lines(1000)
{
}

Lines::~Lines()
{
}

void 
Lines::GetParameters(Parameters *params) const 
{
    Field::GetParameters(params);

    if(params)
    {
        params->AddParameter("number of lines", 1, 10000000, m_number_of_lines);

/*
    , m_threshold(.3)
    , m_size(30)
    , m_fine(true)
    , m_contour(true)
*/
    }
}

// Apply parameters to a processor.
void
Lines::SetParameters(const Parameters *params)
{
    Field::SetParameters(params);
    if(params)
    {
        params->GetParameter("number of lines", m_number_of_lines);
    }
}

spectral::Image *
Lines::Process(void)
{
    spectral::Image *result = NULL;
    spectral::image<float> *hits = NULL;

    Progress(0);

    if(m_picture && m_field && 
      (m_picture->get_width() == m_field->get_width()) &&
      (m_picture->get_height() == m_field->get_height()))
    {
        result = new spectral::Image(*m_picture);
        hits = new spectral::image<float>(m_picture->get_width(),
                                          m_picture->get_height(), 1);
    }

    if(result && hits)
    {
        int i = 0;
        int width = m_picture->get_width();
        int height = m_picture->get_height();
        float val(0.0);

        //std::cerr << "Making vector field" << std::endl;
        spectral::image<float> *field = MakeField();

        //std::cerr << "Initialise weights" << std::endl;
        for(int iy=0;iy<height;iy++)
        {
            for(int ix=0;ix<width;ix++)
            {
                float pixel[3];
                {
                    const float *tmp;
                    tmp = m_picture->get_pixel(ix, iy);
                    pixel[0] = tmp[0];
                    pixel[1] = tmp[1];
                    pixel[2] = tmp[2];
                }
                xyy_to_xyz(pixel);
                result->set_pixel(ix, iy, pixel);
                hits->set_pixel(ix, iy, &val);
            }
        }
        for(i=0;i<m_number_of_lines;i++)
        {
            float x, y;

            x = halton(5, i);
            y = halton(3, i);
            x*= width;
            y*= height;

            if((i % 1000) == 0)
            {
                float progress = i;
                progress*= 100.0;
                progress/= m_number_of_lines;
                Progress(progress);
            }

            {
                std::list<std::pair<float, float> > line;
                line.clear();

                BuildStreamline(x, y, field, line);

                if(line.size())
                {
                    DarkStreamline(line, result, hits);
                }
            }
        }
        for(int iy=0;iy<height;iy++)
        {
            for(int ix=0;ix<width;ix++)
            {
                float pixel[3];
                const float *tmp;

                tmp = result->get_pixel(ix, iy);

                pixel[0] = tmp[0];
                pixel[1] = tmp[1];
                pixel[2] = tmp[2];

                xyz_to_xyy(pixel);

                tmp = hits->get_pixel(ix, iy);

                if(*tmp > 1.0e-6)
                    pixel[2]/= *tmp;
                else
                    std::cerr << "Error" << ix << ", " << iy << "\n";

                result->set_pixel(ix, iy, pixel);
            }
        }
        delete hits;
        delete field;
    }

    Progress(100);
    
    return result;
}

}
