#ifndef __GLOBAL_H__
#define __GLOBAL_H__

#include <vector>

#include <stdint.h>

#include "utils.h"
#include "image.h"
#include "processor.h"
#include "thread.h"

namespace global
{

typedef struct _filter_pixel
{
    uint32_t index;
    float value;
} filter_pixel;
////////////////////////////////////////////////////////////////////////////////

//! Implements a transformation equivalent to a bilateral filter with a 
//! quadratic range weighting function and infinite spatial extent.
class Global : public spectral::Processor
{
public:
    Global();
    ~Global();

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    // Proces image.
    void CrunchImage(const spectral::Image *in, spectral::Image *out, float threshold);

    const spectral::Image *m_lumImage;
    float m_threshold;
};

}

#endif
