#ifndef __DRAGO_H__
#define __DRAGO_H__

#include <stdint.h>
#include "processor.h"

namespace logmap {

class Drago : public spectral::Processor
{
public:
    Drago();
    virtual ~Drago();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    float GetAdaptation(spectral::Image *img, uint32_t num_samples, float &max);

    const spectral::Image *m_picture;

    float m_b;
    float m_exposure;
    int32_t m_samples;
};

}

#endif
