#ifndef __MLFILTER_H__
#define __MLFILTER_H__

#include <vector>

#include <stdint.h>

#include "utils.h"
#include "image.h"
#include "processor.h"
#include "thread.h"

namespace multilevel
{

class MlFilter2;
////////////////////////////////////////////////////////////////////////////////

//! Create a local adaptation image from a luminance channel using an 
//! approximate multi scale gaussian blur.
class MlFilter2 : public spectral::Processor
{
public:
    MlFilter2();
    ~MlFilter2();

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);
    

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    spectral::Image *FastBlur(spectral::Image *img, 
                              spectral::Image **tmp,
                              uint32_t size, uint32_t passes);
    void CalculateWeights(const spectral::Image *reference, 
                          const spectral::Image *image,
                          spectral::Image *result,
                          spectral::Image *weight,
                          float m_tolerance);

    void ApplyWeights(spectral::Image *result,
                      spectral::Image *weight);

    const spectral::Image *m_lumImage;
    std::vector<spectral::Image *>m_layers;
    float   m_tolerance;
    int32_t m_num_layers;
    int32_t m_passes;
    int32_t m_layer_scale;
    bool    m_wrap_horizontal, m_wrap_vertical;
};

}

#endif
