#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <utility>
#include <queue>
#include <vector>

#include "parameters.h"

#include "chroma.h"
#include "bilateral.h"

namespace noise {

const uint32_t IMG_PICTURE = 0;

////////////////////////////////////////////////////////////////////////////////
//
// ChromaNoise processor implementation.
//
////////////////////////////////////////////////////////////////////////////////
ChromaNoise::ChromaNoise()
    : spectral::Processor()
    , m_image(NULL)
    , m_filter_size(10)
    , m_threshold(10)
{
}

ChromaNoise::~ChromaNoise()
{
}

// Get slots & output type.  Returns output colour space.
// pairs are <slot id, colour space>
uint32_t
ChromaNoise::GetInterface(
    std::list<std::pair<uint32_t, uint32_t> > &mandatory,
    std::list<std::pair<uint32_t, uint32_t> > &optional) const
{
    mandatory.clear();
    optional.clear();

    mandatory.push_back(
        std::pair<uint32_t, uint32_t>(IMG_PICTURE, CLS_CIE_XYY));

    return CLS_CIE_XYY;
}

bool
ChromaNoise::SetImage(uint32_t slot, const spectral::Image *img)
{
    if(slot == IMG_PICTURE)
    {
        m_image = NULL;

        if(img)
        {
            if(img->get_colour_space() == CLS_CIE_XYY)
            {
                m_image = img;

                return true;
            }
        }
        else
        {
            return true;
        }
    }
    return false;
}

void
ChromaNoise::GetParameters(spectral::Parameters *params) const
{
    if(params)
    {
        std::string tmp;

        tmp = "filter size";
        params->AddParameter(tmp, 1, 30, m_filter_size);
        
	tmp = "threshold";
        params->AddParameter(tmp, 1, 128, m_threshold);
    }
}

void
ChromaNoise::SetParameters(const spectral::Parameters *params)
{
    if(params)
    {
       std::string tmp;

       tmp = "filter size";
       params->GetParameter(tmp, m_filter_size);

       tmp = "threshold";
       params->GetParameter(tmp, m_threshold);
    }
}

spectral::Image *
ChromaNoise::Process(void)
{
    spectral::Image *result(NULL);

    Progress(0.0);
    if(m_image)
    {
       result = bilateral_filter(m_filter_size, m_threshold, 128,
                                 false, m_image);
    }
    Progress(100.0);
    return result;
}

void
ChromaNoise::ClearImage(uint32_t slot)
{
    if(slot == IMG_PICTURE)
    {
        m_image = NULL;
    }
}

}

