#ifndef __MLFILTER_H__
#define __MLFILTER_H__

#include <vector>

#include <stdint.h>

#include "utils.h"
#include "image.h"
#include "processor.h"
#include "thread.h"

namespace noise
{

class ChromaNoise;
////////////////////////////////////////////////////////////////////////////////

class HPassJob : public thread::Job
{
public:
    HPassJob(const spectral::Image *sourceImage, 
             const std::vector<float> &kernel,
             uint32_t first,
             uint32_t last,
             spectral::Image *destImage);

    virtual void Process(void);

private:
    const spectral::Image *m_sourceImage;
    spectral::Image       *m_destImage;
    const std::vector<float> &m_kernel;
    const uint32_t m_first, m_last;
};

////////////////////////////////////////////////////////////////////////////////

class VPassJob : public thread::Job
{
public:
    VPassJob(const spectral::Image *sourceImage, 
             const std::vector<float> &kernel,
             uint32_t first,
             uint32_t last,
             spectral::Image *destImage);

    virtual void Process(void);

private:
    const spectral::Image *m_sourceImage;
    spectral::Image       *m_destImage;
    const std::vector<float> &m_kernel;
    const uint32_t m_first, m_last;
};

////////////////////////////////////////////////////////////////////////////////

class ChromaNoiseFarmer : public thread::Farmer
{
public:
    ChromaNoiseFarmer(ChromaNoise*,
                   float initialProgress,
                   float jobWeight);
    virtual void ProcessCompletedJob(thread::Job *);
private:
    float m_initialProgress, m_jobWeight;
    ChromaNoise *m_owner;
};


////////////////////////////////////////////////////////////////////////////////

//! Reduce the level of chroma noise in an image.
class ChromaNoise : public spectral::Processor
{
public:
    ChromaNoise();
    ~ChromaNoise();

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    const spectral::Image *m_image;
    int32_t m_filter_size;
    int32_t m_threshold;
};

}

#endif
