#ifndef __PHOTOGRAPHIC_H__
#define __PHOTOGRAPHIC_H__

#include <stdint.h>
#include "processor.h"

namespace photoreceptor {

//! Implements Rheinhard and Devlins "Photoreceptor" tone mapping algorithm.
class Photoreceptor : public spectral::Processor
{
public:
    Photoreceptor();
    virtual ~Photoreceptor();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    void GetAdaptation(spectral::Image *, uint32_t, float &, float &, float &, float &);
    void GetAverageRGB(spectral::Image *, uint32_t, float *, float *);

    // data members
    const spectral::Image *m_picture;

    int32_t m_samples;
    float m_contrast;
    float m_saturation;
    float m_adaptation;
    float m_intensity;
};

}

#endif
