#ifndef __MAPPING_H__
#define __MAPPING_H__

#include <math.h>
#include <stdio.h>
#include "utils.h"

#ifndef PI
#define PI 3.14159
#endif
#ifndef PI_OVER_TWO
#define PI_OVER_TWO (PI/2.0)
#endif

namespace remap {

////////////////////////////////////////////////////////////////////////////////
class Mapping
{
public:
    virtual ~Mapping(){}
    virtual bool MapTo(float lat, float lon, float &x, float &y) const = 0;
    virtual bool MapFrom(float x, float y, float &lat, float &lon) const = 0;

    bool LatLongToVector(float lat, float lon, float *vector) const;
    bool VectorToLatLong(const float *vector, float &lat, float &lon) const;
};


////////////////////////////////////////////////////////////////////////////////
class Equirectangular : public Mapping
{
public:
    Equirectangular(int xres, int yres, float phi1);
    virtual ~Equirectangular();
    virtual bool MapTo(float lat, float lon, float &x, float &y) const;
    virtual bool MapFrom(float x, float y, float &lat, float &lon) const;
private:
    int m_xres, m_yres;
    float m_phi1;
};

////////////////////////////////////////////////////////////////////////////////
class AngularFisheye : public Mapping
{
public:
    AngularFisheye(float centre_x, float centre_y, float radius);
    virtual ~AngularFisheye();
    virtual bool MapTo(float lat, float lon, float &x, float &y) const;
    virtual bool MapFrom(float x, float y, float &lat, float &lon) const;
    
    bool SetZenith(float x, float y);
    void GetCentre(float &x, float &y) const;
    void GetZenith(float &x, float &y) const;
private:
    float m_centre_x, m_centre_y,
          m_zenith_x, m_zenith_y,
          m_radius;
    Matrix33 m_forward_transform, 
             m_inverse_transform;
};

void circumcircle(float, float, float, float, float, float, float&, float&, float&);

////////////////////////////////////////////////////////////////////////////////

}

#endif

