#ifndef __REMAP_H__
#define __REMAP_H__

#include <stdint.h>
#include "processor.h"

namespace remap {

class Remap : public spectral::Processor
{
public:
    Remap();
    virtual ~Remap();

protected:

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t type, const spectral::Image *img);

    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);

    // Process images.
    virtual spectral::Image *Process(void);

    // Clear the image in a slot.
    virtual void ClearImage(uint32_t type);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    void GetBackground(float x, float y, float r,  float coverage,
                       uint32_t samples,
                       const spectral::Image *img,
                       float *background);


    // data members
    const spectral::Image *m_picture;

    float m_x1, m_y1, m_x2, m_y2, m_x3, m_y3;
    float m_zenith_x, m_zenith_y;
    float m_phi1;
    bool  m_auto_width;
    bool  m_move_zenith;
    int   m_width;
    int   m_samples;
    int   m_bg_cover;
    uint32_t m_filter;
};

}

#endif
