#include <stdio.h>
#include <stdlib.h>

#include "mapping.h"


#ifndef PI
#define PI 3.14159265
#endif

bool test(const char *name,
          float min_lat, float max_lat, 
          float min_long, float max_long,
          Mapping *map)
{
    const float error = 1.0e-4;
    bool success = true;

    if(map)
    {
        printf("\nTesting \"%s\":\n", name);
        for(int i=0;i<10;i++)
        {
            float lat_in, long_in, lat_out, long_out, x, y;

            lat_in = rand();
            lat_in/= RAND_MAX;
            lat_in*= (max_lat - min_lat);
            lat_in+= min_lat;

            long_in = rand();
            long_in/= RAND_MAX;

            long_in*= (max_long - min_long);
            long_in+= min_long;

            if(map->MapTo(lat_in, long_in, x, y))
            {
                if(map->MapFrom(x, y, lat_out, long_out))
                {
                    if((fabs(lat_out - lat_in) > error) ||
                       (fabs(long_out - long_in) > error))
                    {
                        printf("ERROR: %f, %f -> %f, %f -> %f, %f\n", 
                               lat_in, long_in, x, y, lat_out, long_out);
                        success = false;
                    }
                }
                else
                {
                    printf("ERROR: MapFrom %f, %f failed\n", x, y);
                    success = false;
                }
            }
            else
            {
                printf("ERROR: MapTo %f, %f failed\n", lat_in, long_in);
                success = false;
            }
        }
    }
    if(success)
    {
        printf("Succeeded\n");
    }
    return success;
}

int main(int argc, char **argv)
{
    bool success = true;

    Mapping *map;
    map = new Equirectangular(1024, 512, 0);
    success&= test("Equirectangular", -(PI / 2.0), (PI / 2.0), -PI, PI, map);
    delete map;

    map = new AngularFisheye(100, 100, 100);
    success&= test("AngularFisheye", 0, (PI / 2.0), -PI, PI, map);
    delete map;

    if(success)
    {
        printf("\nAll tests succeeded.\n");
    }
    else
    {
        printf("\nSome tests failed.\n");
    }

    return 0;
}
