#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "utils.h"
#include "thread.h"
#include "image.h"
#include "parameters.h"

#include "replacelum.h"

const uint32_t IMG_PICTURE = 0;
const uint32_t IMG_LUM = 1;

namespace replacelum {

ReplaceLuminance::ReplaceLuminance()
    : spectral::Processor()
    , m_picture(NULL)
    , m_lum(NULL)
{
}

ReplaceLuminance::~ReplaceLuminance()
{
}

// Get slots & output type.  Returns output colour space.
// pairs are <slot id, colour space>
uint32_t
ReplaceLuminance::GetInterface(
    std::list<std::pair<uint32_t, uint32_t> > &mandatory,
    std::list<std::pair<uint32_t, uint32_t> > &optional) const
{   
    mandatory.clear();
    optional.clear();
        
    mandatory.push_back(     
        std::pair<uint32_t, uint32_t>(IMG_PICTURE, CLS_CIE_XYY));
            
    mandatory.push_back(     
        std::pair<uint32_t, uint32_t>(IMG_LUM, CLS_LUMINANCE));

    return CLS_CIE_XYY;
}  

bool
ReplaceLuminance::SetImage(uint32_t type, const spectral::Image *img)
{
    bool result = false;

    if(type == IMG_PICTURE)
    {
        m_picture = NULL;
        if(img)
        {
            if(img->get_colour_space() == CLS_CIE_XYY)
            {
                m_picture = img;
                result = true;
            }
        }
        else
        {
            result = true;
        }
    }

    if(type == IMG_LUM)
    {
        m_lum = NULL;
        if(img)
        {
            if(img->get_colour_space() == CLS_LUMINANCE)
            {
                m_lum = img;
                result = true;
            }
        }
        else
        {
            result = true;
        }
    }

    return result;
}

void
ReplaceLuminance::ClearImage(uint32_t type)
{
    if(type == IMG_PICTURE)
    {
        m_picture = NULL;
    }
    if(type == IMG_LUM)
    {
        m_lum = NULL;
    }
}

void 
ReplaceLuminance::GetParameters(spectral::Parameters *) const 
{
}

// Apply parameters to a processor.
void
ReplaceLuminance::SetParameters(const spectral::Parameters *)
{
}

spectral::Image *
ReplaceLuminance::Process(void)
{
    spectral::Image *result = NULL;

    Progress(0);

    if(m_picture && 
       m_lum &&
       (m_picture->get_width() == m_lum->get_width()) &&
       (m_picture->get_height() == m_lum->get_height()))
    {
        result = new spectral::Image(*m_picture);
    }

    if(result)
    {
        size_t size = size_t(result->get_width()) * size_t(result->get_height());
        size_t i(0), j(2);
        while(i < size)
        {
            result->get_buffer()[j] = m_lum->get_buffer()[i];
            if(i%10000 == 0)
            {
                float tmp = i * 100;
                tmp/= size;
                Progress(tmp);
            }
            i++;
            j+=3;
        }
        Progress(100);
    }
    return result;
}

}
