#ifndef __YEE_H__
#define __YEE_H__

#include <vector>

#include <stdint.h>

#include "processor.h"

namespace segment
{

//! Create a local adaptation luminance image from a luminance channel by
//! segmentation.
class Yee : public spectral::Processor
{
public:
    Yee();
    ~Yee();

    // Get slots & output type.  Returns output colour space.
    // pairs are <slot id, colour space>
    virtual uint32_t GetInterface(
        std::list<std::pair<uint32_t, uint32_t> > &mandatory,
        std::list<std::pair<uint32_t, uint32_t> > &optional) const;

    // Attempt to set an image in a slot.
    virtual bool SetImage(uint32_t slot, const spectral::Image *img);
    
    // Apply parameters to a processor.
    virtual void SetParameters(const spectral::Parameters *);
    
    // Process images.
    virtual spectral::Image *Process(void);
    
    // Clear the image in a slot.
    virtual void ClearImage(uint32_t slot);

protected:

    // Populate a spectral::Parameters object.
    virtual void GetParameters(spectral::Parameters *) const;

private:
    private:
    float GetBinSize(int id);

    spectral::Image *m_lumImage;
    float  m_min, m_max;
    int32_t m_first_layer_id, m_last_layer_id;
    uint32_t m_pattern;
    bool m_wrap_horizontal, m_wrap_vertical;
};

}

#endif
