#include <string>
#include <stdio.h>
#include <stdint.h>
#include <math.h>

#include <ImfArray.h>
#include <ImfRgba.h>
#include <ImfRgbaFile.h>
#include <ImfFloatAttribute.h>
#include <ImfHeader.h>

#include "utils.h"
#include "bufutils.h"
#include "exr_saver.h"

const float MAX_OUTPUT_LUMINANCE = 512.0;

bool
ExrSaver::SaveImage(const spectral::Image *image, const std::string &fn)
{
    bool result(false);

    if(image && image->get_buffer() && fn.length())
    {
        uint32_t width = image->get_width();
        uint32_t height = image->get_height();
	size_t size = size_t(width) * size_t(height);
	Imf::Rgba *pixels = NULL;
        float stonits = 1.0;

        {
	    float max = 0.0;
	    float *buf = image->get_buffer();
	    size_t i, idx;

	    for(i=0,idx=2;i<size;i++,idx+=3)
	    {
	        if(buf[i] > max)
		{
		    max = buf[i];
		}
	    }
	    if(max > MAX_OUTPUT_LUMINANCE)
	    {
	        stonits = max / MAX_OUTPUT_LUMINANCE;
	    }
	}
        try
	{
            // Create header with stonits info.
	    Imf::Header header(width, height);
	    header.insert("whiteLuminance", Imf::FloatAttribute(stonits));

	    // Create file.
            Imf::RgbaOutputFile file(fn.c_str(), header,
	                             Imf::WRITE_RGBA);

            // Create frame buffer.
            pixels = new Imf::Rgba[width * height];

            if(pixels)
	    {
	        float *buf = image->get_buffer();
                size_t idx = 0;
		for(uint32_t y=0;y<height;y++)
		{
		    for(uint32_t x=0;x<width;x++)
		    {
		        float pixel[3];
			pixel[0] = buf[0];
			pixel[1] = buf[1];
			pixel[2] = buf[2]/stonits;

			xyy_to_linear_srgb(pixel);
			pixels[idx].r = half(pixel[0]);
			pixels[idx].g = half(pixel[1]);
			pixels[idx].b = half(pixel[2]);
                        pixels[idx].a = half(1.0);

                        buf+=3;
			idx++;
		    }
	        }
	        file.setFrameBuffer(pixels, 1, width);
	        file.writePixels(height);
	    }
	    result = true;
            delete pixels;
        }
	catch(...)
	{
	    std::cerr << "Failed to save file \"" << fn << "\"" << std::endl;
            if(pixels)
	    {
	        delete pixels;
	    }
        }
    }
    return result;
}
