#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <string.h>

#include "utils.h"
#include "htond.h"
#include "bufutils.h"
#include "pfm_saver.h"

float swap_bytes(float in)
{
    float result;
    unsigned char *p1, *p2;

    p1 = (unsigned char *)(&in);
    p2 = (unsigned char *)(&result);

    p2[0]=p1[3];
    p2[1]=p1[2];
    p2[2]=p1[1];
    p2[3]=p1[0];

    return result;
}

bool
PfmSaver::SaveImage(const spectral::Image *image, const std::string &fn)
{
    bool result(false);
    FILE *fp;

    if(image && image->get_width() && image->get_height())
    {
        fp = fopen(fn.c_str(), "wb");

        if(fp)
        {
            size_t pixel_size(0);
            if(image->get_colour_space() == CLS_CIE_XYY)
            {
                pixel_size = 3;
            }
            else if(image->get_colour_space() == CLS_LUMINANCE)
            {
                pixel_size = 1;
            }

            if(pixel_size)
            {
                float *line;
                uint32_t y;
                size_t line_size = size_t(image->get_width()) *
                                   size_t(pixel_size);
                line = (float *)calloc(line_size, sizeof(float));

                switch(pixel_size)
                {
                case 1:
                    fprintf(fp, "Pf\n");
                    break;
                case 3:
                    fprintf(fp, "PF\n");
                    break;
                }

                fprintf(fp, "%i %i\n", image->get_width(), image->get_height());
                fprintf(fp, "-1.0\n");

                y = image->get_height();
                
                do 
                {
                    size_t i;
                    float *p;
                    y--;
                    p = (image->get_buffer() + (y * line_size));

                    memcpy(line, p, line_size * sizeof(float));
                    
                    if(image->get_colour_space() == CLS_CIE_XYY)
                    {
                        buf_xyy_to_linear_srgb(line, image->get_width());
                    }

                    for(i=0;i<line_size;i++)
                    {
                        line[i] = swap_bytes(sp_htonf(line[i]));
                    }

                    fwrite(line, sizeof(float), line_size, fp);
                } while(y);

                free((void *)line);
                
            }

            fclose(fp);
        }
    }


    return result;
}
