#include <png.h>
#include <string>
#include <stdio.h>
#include <stdint.h>
#include <math.h>

#include "utils.h"
#include "htond.h"
#include "png_saver.h"

//#define WRITE_16BIT

#ifdef WRITE_16BIT
typedef unsigned short PIXEL_TYPE;
#define STM_PNG_BITS 16
#define STM_PNG_MASK 0xffff
#define STM_PNG_MAX  65535
#else
typedef unsigned char PIXEL_TYPE;
#define STM_PNG_BITS 8
#define STM_PNG_MASK 0xff
#define STM_PNG_MAX  255
#endif

bool
PngSaver::SaveImage(const spectral::Image *image, const std::string &fn)
{
    bool result(false);
    FILE *fp;

    if(image)
    {
        fp = fopen(fn.c_str(), "wb");
        
        if(fp)
        {
            png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
                                                          NULL,
                                                          NULL,
                                                          NULL);
    
            if(png_ptr)
            {
                png_infop info_ptr = png_create_info_struct(png_ptr);
                if(info_ptr)
                {
                    if (setjmp(png_jmpbuf(png_ptr)))
                    {
                        fclose(fp);
                    }
                    else
                    {
                        png_init_io(png_ptr, fp);
                        uint32_t width(0), height(0), cs(0);
                        
                        const float *image_data = image->get_buffer();
                        width = image->get_width();
                        height = image->get_height();
                        cs = image->get_colour_space();
 
                        if((cs == CLS_SRGB) ||
                           (cs == CLS_SRGB_LINEAR) ||
                           (cs == CLS_CIE_XYY)) 
                        {
                            png_set_IHDR(png_ptr, info_ptr,
                                         width, height, STM_PNG_BITS,
                                         PNG_COLOR_TYPE_RGB,
                                         PNG_INTERLACE_NONE,
                                         PNG_COMPRESSION_TYPE_DEFAULT,
                                         PNG_FILTER_TYPE_DEFAULT);
        
                            png_set_sRGB(png_ptr, info_ptr, 
                                         PNG_sRGB_INTENT_ABSOLUTE);
        
                            png_set_filter(png_ptr, PNG_FILTER_TYPE_BASE, 
                                           PNG_FILTER_SUB);
                            png_set_compression_level(png_ptr, Z_BEST_SPEED);
                            png_set_compression_strategy(png_ptr, Z_HUFFMAN_ONLY);
        
                            png_write_info(png_ptr, info_ptr);
        
                            PIXEL_TYPE *row;
                            row = (PIXEL_TYPE *)malloc(width * 3 * sizeof(PIXEL_TYPE));
                            
                            if(row)
                            {
                                png_bytep row_ptr = (png_bytep)row;
                                size_t size = size_t(width) * size_t(height) * 3;
                                size_t row_index = 0;
                                size_t row_size  = size_t(width) * 3;
                                size_t index = 0;
    
                                while(index < size)
                                { 
                                    int n;
                                    float fval;
                                    float px[3];
                                    if((index % 3) == 0)
                                    {
                                        for(uint32_t j=0;j<3;j++)
                                        {
                                            px[j] = image_data[index + j];
                                        }
                                        if(cs == CLS_CIE_XYY)
                                        {
                                            xyy_to_linear_srgb(px);
                                            px[0] = apply_srgb_gamma(px[0]);
                                            px[1] = apply_srgb_gamma(px[1]);
                                            px[2] = apply_srgb_gamma(px[2]);
                                        }
                                        else if(cs == CLS_SRGB_LINEAR)
                                        {
                                            px[0] = apply_srgb_gamma(px[0]);
                                            px[1] = apply_srgb_gamma(px[1]);
                                            px[2] = apply_srgb_gamma(px[2]);
                                        }
                                        else if(cs == CLS_SRGB)
                                        {
                                            //noop
                                        }
                                    }
                                    fval = px[index % 3];
                                    index++;
    
                                    if(fval < 0.0) fval = 0.0;
                                    if(fval > 1.0) fval = 1.0;
                                    n = (int)(fval * STM_PNG_MAX);
                                    if(n < 0) n = 0;
                                    if(n > STM_PNG_MAX) n = STM_PNG_MAX;
#ifdef WRITE_16BIT

                                    n = sp_htons(n);
#endif
                                    row[row_index++] = (n & STM_PNG_MASK);
    
                                    if(row_index == row_size)
                                    {
                                        row_index = 0;
                                        png_write_row(png_ptr, row_ptr);
                                    }
                                }
                                result = true;
    			        free((void *)row);
                            }
                            png_write_end(png_ptr, info_ptr);
                        }
                    }
                }
                png_destroy_write_struct(&png_ptr, &info_ptr);
            }
            fclose(fp);
        }
    }
    return result;
}
