#include <stdio.h>
#include <string.h>

void print_include(void)
{
    printf("#include \"htond.h\"\n\n");
}

void print_double_body(const int *from, const int *to)
{
    int i;

    printf("{\n");
    printf("    double out;\n");
    printf("    unsigned char *from, *to;\n");
    printf("    from = (unsigned char *)(&in);\n");
    printf("    to = (unsigned char *)(&out);\n");

    for(i=0;i<8;i++)
    {
        printf("    to[%i] = from[%i];\n", to[i], from[i]);
    }
    printf("    return out;\n");
    printf("}\n");
}

void print_double(void)
{
    char buf[8];
    int  location[8];
    int  seen[8];
    int i;
    const double magic[8] = {
        7.410984687618698162649e-323, /* 000000000000000F */
        2.023198819719904598403e-320, /* 0000000000000FFF */
        5.180648845879850950599e-318, /* 00000000000FFFFF */
        1.326247364412638738532e-315, /* 000000000FFFFFFF */
        3.395193265495029139594e-313, /* 0000000FFFFFFFFF */
        8.691694759793261337050e-311, /* 00000FFFFFFFFFFF */
        2.225073858507200889025e-308, /* 000FFFFFFFFFFFFF */
        1.288229753919426518623e-231  /* 0FFFFFFFFFFFFFFF */
    };
    const int be_location[8] = {7, 6, 5, 4, 3, 2, 1, 0};

    for(i=0;i<8;i++)
    {
        buf[i]   = 0;
        location[i] = -1;
        seen[i]  = 0;
    }

    for(i=0;i<8;i++)
    {
        int j;
        int found;
        memcpy(buf, &(magic[i]), 8);
        found = 0;

        for(j=0;j<8;j++)
        {
            if(buf[j] && !(seen[j]))
            {
                seen[j] = 1;
                location[i] = j;
                found = 1;
                break;
            }
        }
        if(!found)
        {
            fprintf(stderr, "Failed to get byte location!\n");
            return;
        }
    }
    {
        int is_be = 1;

        for(i=0;i<8;i++)
        {
            /*fprintf(stderr, "location of byte %i is %i\n", i, location[i]);*/
            if(location[i] != be_location[i])
            {
                is_be = 0;
            }
        }
        

        if(is_be)
        {
            printf("double sp_htond(double in){return in;}\n");
            printf("double sp_ntohd(double in){return in;}\n");
        }
        else
        {
            printf("double sp_htond(double in)\n");
            print_double_body(location, be_location);
            printf("\n\ndouble sp_ntohd(double in)\n");
            print_double_body(be_location, location);
        }
    }
}

void print_float_body(const int *from, const int *to)
{
    int i;

    printf("{\n");
    printf("    float out;\n");
    printf("    unsigned char *from, *to;\n");
    printf("    from = (unsigned char *)(&in);\n");
    printf("    to = (unsigned char *)(&out);\n");

    for(i=0;i<4;i++)
    {
        printf("    to[%i] = from[%i];\n", to[i], from[i]);
    }
    printf("    return out;\n");
    printf("}\n");

}

void print_float(void)
{
    char buf[4];
    int  location[4];
    int  seen[4];
    int i;
    const float magic[4] = {
        2.10194769649e-44, /* 0000000F */
        5.73831721141e-42, /* 00000FFF */
        1.46936653723e-39, /* 000FFFFF */
        2.52435474624e-29, /* 0FFFFFFF */
    };
    const int be_location[4] = {3, 2, 1, 0};

    for(i=0;i<4;i++)
    {
        buf[i]   = 0;
        location[i] = -1;
        seen[i]  = 0;
    }

    for(i=0;i<4;i++)
    {
        int j;
        int found;
        memcpy(buf, &(magic[i]), 4);
        found = 0;

        for(j=0;j<4;j++)
        {
            if(buf[j] && !(seen[j]))
            {
                seen[j] = 1;
                location[i] = j;
                found = 1;
                break;
            }
        }
        if(!found)
        {
            fprintf(stderr, "Failed to get byte location!\n");
            return;
        }
    }
    {
        int is_be = 1;

        for(i=0;i<4;i++)
        {
            /*fprintf(stderr, "location of byte %i is %i\n", i, location[i]);*/
            if(location[i] != be_location[i])
            {
                is_be = 0;
            }
        }
        

        if(is_be)
        {
            printf("float sp_htonf(float in){return in;}\n");
            printf("float sp_ntohf(float in){return in;}\n");
        }
        else
        {
            printf("float sp_htonf(float in)\n");
            print_float_body(location, be_location);
            printf("\n\nfloat sp_ntohf(float in)\n");
            print_float_body(be_location, location);
        }
    }
}


void print_long_body(const int *from, const int *to)
{
    int i;

    printf("{\n");
    printf("    unsigned long out;\n");
    printf("    unsigned char *from, *to;\n");
    printf("    from = (unsigned char *)(&in);\n");
    printf("    to = (unsigned char *)(&out);\n");

    for(i=0;i<4;i++)
    {
        printf("    to[%i] = from[%i];\n", to[i], from[i]);
    }
    printf("    return out;\n");
    printf("}\n");
}

void print_long(void)
{
    char buf[4];
    int  location[4];
    int  seen[4];
    int i;
    const unsigned long magic[4] = {
        0x000000ff,
        0x0000ff00,
        0x00ff0000,
        0xff000000,
    };
    const int be_location[4] = {3, 2, 1, 0};

    for(i=0;i<4;i++)
    {
        buf[i]   = 0;
        location[i] = -1;
        seen[i]  = 0;
    }

    for(i=0;i<4;i++)
    {
        int j;
        int found;
        memcpy(buf, &(magic[i]), 4);
        found = 0;

        for(j=0;j<4;j++)
        {
            if(buf[j] && !(seen[j]))
            {
                seen[j] = 1;
                location[i] = j;
                found = 1;
                break;
            }
        }
        if(!found)
        {
            fprintf(stderr, "Failed to get byte location!\n");
            return;
        }
    }
    {
        int is_be = 1;

        for(i=0;i<4;i++)
        {
            /*fprintf(stderr, "location of byte %i is %i\n", i, location[i]);*/
            if(location[i] != be_location[i])
            {
                is_be = 0;
            }
        }
        

        if(is_be)
        {
            printf("unsigned long sp_htonl(unsigned long in){return in;}\n");
            printf("unsigned long sp_ntohl(unsigned long in){return in;}\n");
        }
        else
        {
            printf("unsigned long sp_htonl(unsigned long in)\n");
            print_long_body(location, be_location);
            printf("\n\nunsigned long sp_ntohl(unsigned long in)\n");
            print_long_body(be_location, location);
        }
    }
}

void print_short_body(const int *from, const int *to)
{
    int i;

    printf("{\n");
    printf("    unsigned short out;\n");
    printf("    unsigned char *from, *to;\n");
    printf("    from = (unsigned char *)(&in);\n");
    printf("    to = (unsigned char *)(&out);\n");

    for(i=0;i<2;i++)
    {
        printf("    to[%i] = from[%i];\n", to[i], from[i]);
    }
    printf("    return out;\n");
    printf("}\n");
}

void print_short(void)
{
    char buf[2];
    int  location[2];
    int  seen[2];
    int i;
    const unsigned long magic[2] = {
        0x000000ff,
        0x0000ff00,
    };
    const int be_location[4] = {1, 0};

    for(i=0;i<2;i++)
    {
        buf[i]   = 0;
        location[i] = -1;
        seen[i]  = 0;
    }

    for(i=0;i<2;i++)
    {
        int j;
        int found;
        memcpy(buf, &(magic[i]), 2);
        found = 0;

        for(j=0;j<2;j++)
        {
            if(buf[j] && !(seen[j]))
            {
                seen[j] = 1;
                location[i] = j;
                found = 1;
                break;
            }
        }
        if(!found)
        {
            fprintf(stderr, "Failed to get byte location!\n");
            return;
        }
    }
    {
        int is_be = 1;

        for(i=0;i<2;i++)
        {
            /*fprintf(stderr, "location of byte %i is %i\n", i, location[i]);*/
            if(location[i] != be_location[i])
            {
                is_be = 0;
            }
        }
        

        if(is_be)
        {
            printf("unsigned short sp_htons(unsigned short in){return in;}\n");
            printf("unsigned short sp_ntohs(unsigned short in){return in;}\n");
        }
        else
        {
            printf("unsigned short sp_htons(unsigned short in)\n");
            print_short_body(location, be_location);
            printf("\n\nunsigned short sp_ntohs(unsigned short in)\n");
            print_short_body(be_location, location);
        }
    }
}

int main(int argc, char **argv)
{
    print_include();
    print_double();
    print_float();
    print_long();
    print_short();
    return 0;
}

